# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_broadcast.proto
# plugin: python-betterproto

from dataclasses import dataclass
from typing import List

import betterproto


class EBroadcastWatchLocation(betterproto.Enum):
    Invalid = 0
    SteamTV_Tab = 1
    SteamTV_WatchParty = 2
    Chat_Tab = 3
    Chat_WatchParty = 4
    CommunityPage = 5
    StoreAppPage = 6
    InGame = 7
    BigPicture = 8
    SalesPage = 9
    CuratorPage = 10
    DeveloperPage = 11
    Chat_Friends = 12
    SteamTV_Web = 13


class EBroadcastChatPermission(betterproto.Enum):
    Public = 0
    OwnsApp = 1


class CBroadcast_WatchBroadcast_ResponseEWatchResponse(betterproto.Enum):
    Ready = 1
    NotAvailable = 2
    WaitingForApproval = 3
    WaitingForStart = 4
    InvalidSession = 5
    TooManyBroadcasts = 6
    WaitingForReconnect = 7
    SystemNotSupported = 8
    UserRestricted = 9
    ClientOutOfDate = 10
    PoorUploadQuality = 11
    MissingSubscription = 12


class CBroadcast_BroadcastViewerState_NotificationEViewerState(betterproto.Enum):
    rNeedsApproval = 1
    rWatching = 2
    rLeft = 3


@dataclass
class CBroadcast_BeginBroadcastSession_Request(betterproto.Message):
    permission: int = betterproto.int32_field(1)
    gameid: int = betterproto.uint64_field(2)
    client_instance_id: int = betterproto.uint64_field(3)
    title: str = betterproto.string_field(4)
    cellid: int = betterproto.uint32_field(5)
    rtmp_token: int = betterproto.uint64_field(6)
    thumbnail_upload: bool = betterproto.bool_field(7)
    client_beta: str = betterproto.string_field(8)
    sysid: int = betterproto.uint32_field(9)


@dataclass
class CBroadcast_BeginBroadcastSession_Response(betterproto.Message):
    broadcast_id: float = betterproto.fixed64_field(1)
    thumbnail_upload_address: str = betterproto.string_field(2)
    thumbnail_upload_token: str = betterproto.string_field(3)
    thumbnail_interval_seconds: int = betterproto.uint32_field(4)
    heartbeat_interval_seconds: int = betterproto.uint32_field(5)


@dataclass
class CBroadcast_EndBroadcastSession_Request(betterproto.Message):
    broadcast_id: float = betterproto.fixed64_field(1)


@dataclass
class CBroadcast_EndBroadcastSession_Response(betterproto.Message):
    pass


@dataclass
class CBroadcast_StartBroadcastUpload_Request(betterproto.Message):
    broadcast_id: float = betterproto.fixed64_field(1)
    cellid: int = betterproto.uint32_field(2)
    as_rtmp: bool = betterproto.bool_field(3)
    delay_seconds: int = betterproto.uint32_field(4)
    rtmp_token: int = betterproto.uint64_field(5)
    upload_ip_address: int = betterproto.uint32_field(6)
    is_replay: bool = betterproto.bool_field(7)
    sysid: int = betterproto.uint32_field(8)


@dataclass
class CBroadcast_StartBroadcastUpload_Response(betterproto.Message):
    upload_token: str = betterproto.string_field(1)
    upload_address: str = betterproto.string_field(2)
    broadcast_upload_id: float = betterproto.fixed64_field(3)
    enable_replay: bool = betterproto.bool_field(6)
    http_address: str = betterproto.string_field(7)


@dataclass
class CBroadcast_NotifyBroadcastUploadStop_Notification(betterproto.Message):
    broadcast_upload_id: float = betterproto.fixed64_field(1)
    upload_result: int = betterproto.uint32_field(2)


@dataclass
class CBroadcast_WatchBroadcast_Request(betterproto.Message):
    steamid: float = betterproto.fixed64_field(1)
    existing_broadcast_id: float = betterproto.fixed64_field(2)
    viewer_token: float = betterproto.fixed64_field(3)
    client_ip: int = betterproto.uint32_field(4)
    client_cell: int = betterproto.uint32_field(5)
    watch_location: "EBroadcastWatchLocation" = betterproto.enum_field(6)
    is_webrtc: bool = betterproto.bool_field(7)


@dataclass
class CBroadcast_WatchBroadcast_Response(betterproto.Message):
    response: "CBroadcast_WatchBroadcast_ResponseEWatchResponse" = betterproto.enum_field(1)
    mpd_url: str = betterproto.string_field(2)
    broadcast_id: float = betterproto.fixed64_field(3)
    gameid: int = betterproto.uint64_field(4)
    title: str = betterproto.string_field(5)
    num_viewers: int = betterproto.uint32_field(6)
    permission: int = betterproto.int32_field(7)
    is_rtmp: bool = betterproto.bool_field(8)
    seconds_delay: int = betterproto.int32_field(9)
    viewer_token: float = betterproto.fixed64_field(10)
    hls_m3u8_master_url: str = betterproto.string_field(11)
    heartbeat_interval: int = betterproto.int32_field(12)
    thumbnail_url: str = betterproto.string_field(13)
    is_webrtc: bool = betterproto.bool_field(14)
    webrtc_session_id: float = betterproto.fixed64_field(15)
    webrtc_offer_sdp: str = betterproto.string_field(16)
    webrtc_turn_server: str = betterproto.string_field(17)
    is_replay: bool = betterproto.bool_field(18)
    duration: int = betterproto.int32_field(19)


@dataclass
class CBroadcast_HeartbeatBroadcast_Notification(betterproto.Message):
    steamid: float = betterproto.fixed64_field(1)
    broadcast_id: float = betterproto.fixed64_field(2)
    viewer_token: float = betterproto.fixed64_field(3)
    representation: int = betterproto.uint32_field(4)


@dataclass
class CBroadcast_StopWatchingBroadcast_Notification(betterproto.Message):
    steamid: float = betterproto.fixed64_field(1)
    broadcast_id: float = betterproto.fixed64_field(2)
    viewer_token: float = betterproto.fixed64_field(3)


@dataclass
class CBroadcast_GetBroadcastStatus_Request(betterproto.Message):
    steamid: float = betterproto.fixed64_field(1)
    broadcast_id: float = betterproto.fixed64_field(2)


@dataclass
class CBroadcast_GetBroadcastStatus_Response(betterproto.Message):
    gameid: int = betterproto.uint64_field(1)
    title: str = betterproto.string_field(2)
    num_viewers: int = betterproto.uint32_field(3)
    permission: int = betterproto.int32_field(4)
    is_rtmp: bool = betterproto.bool_field(5)
    seconds_delay: int = betterproto.int32_field(6)
    is_publisher: bool = betterproto.bool_field(7)
    thumbnail_url: str = betterproto.string_field(8)
    update_interval: int = betterproto.int32_field(9)
    is_uploading: bool = betterproto.bool_field(10)
    duration: int = betterproto.uint32_field(11)
    is_replay: bool = betterproto.bool_field(12)
    is_capturing_vod: bool = betterproto.bool_field(13)
    is_store_whitelisted: bool = betterproto.bool_field(14)


@dataclass
class CBroadcast_GetBroadcastThumbnail_Request(betterproto.Message):
    steamid: float = betterproto.fixed64_field(1)
    broadcast_id: float = betterproto.fixed64_field(2)


@dataclass
class CBroadcast_GetBroadcastThumbnail_Response(betterproto.Message):
    thumbnail_url: str = betterproto.string_field(1)
    update_interval: int = betterproto.int32_field(2)
    num_viewers: int = betterproto.int32_field(3)
    duration: int = betterproto.int32_field(4)


@dataclass
class CBroadcast_InviteToBroadcast_Request(betterproto.Message):
    steamid: float = betterproto.fixed64_field(1)
    approval_response: bool = betterproto.bool_field(2)


@dataclass
class CBroadcast_InviteToBroadcast_Response(betterproto.Message):
    success: bool = betterproto.bool_field(1)


@dataclass
class CBroadcast_SendBroadcastStateToServer_Request(betterproto.Message):
    permission: int = betterproto.int32_field(1)
    gameid: int = betterproto.uint64_field(2)
    title: str = betterproto.string_field(3)
    game_data_config: str = betterproto.string_field(4)


@dataclass
class CBroadcast_SendBroadcastStateToServer_Response(betterproto.Message):
    pass


@dataclass
class CBroadcast_NotifyBroadcastSessionHeartbeat_Notification(betterproto.Message):
    broadcast_id: float = betterproto.fixed64_field(1)


@dataclass
class CBroadcast_GetBroadcastChatInfo_Request(betterproto.Message):
    steamid: float = betterproto.fixed64_field(1)
    broadcast_id: float = betterproto.fixed64_field(2)
    client_ip: int = betterproto.uint32_field(3)
    client_cell: int = betterproto.uint32_field(4)


@dataclass
class CBroadcast_GetBroadcastChatInfo_Response(betterproto.Message):
    chat_id: float = betterproto.fixed64_field(1)
    view_url_template: str = betterproto.string_field(3)
    flair_group_ids: List[int] = betterproto.uint32_field(4)


@dataclass
class CBroadcast_PostChatMessage_Request(betterproto.Message):
    chat_id: float = betterproto.fixed64_field(1)
    message: str = betterproto.string_field(2)
    instance_id: int = betterproto.uint32_field(3)
    language: int = betterproto.uint32_field(4)
    country_code: str = betterproto.string_field(5)


@dataclass
class CBroadcast_PostChatMessage_Response(betterproto.Message):
    persona_name: str = betterproto.string_field(1)
    in_game: bool = betterproto.bool_field(2)
    result: int = betterproto.int32_field(3)
    cooldown_time_seconds: int = betterproto.int32_field(4)


@dataclass
class CBroadcast_UpdateChatMessageFlair_Request(betterproto.Message):
    chat_id: float = betterproto.fixed64_field(1)
    flair: str = betterproto.string_field(2)


@dataclass
class CBroadcast_UpdateChatMessageFlair_Response(betterproto.Message):
    result: int = betterproto.int32_field(1)
    chat_id: float = betterproto.fixed64_field(2)
    flair: str = betterproto.string_field(3)


@dataclass
class CBroadcast_MuteBroadcastChatUser_Request(betterproto.Message):
    chat_id: float = betterproto.fixed64_field(1)
    user_steamid: float = betterproto.fixed64_field(2)
    muted: bool = betterproto.bool_field(3)


@dataclass
class CBroadcast_MuteBroadcastChatUser_Response(betterproto.Message):
    pass


@dataclass
class CBroadcast_RemoveUserChatText_Request(betterproto.Message):
    chat_id: float = betterproto.fixed64_field(1)
    user_steamid: float = betterproto.fixed64_field(2)


@dataclass
class CBroadcast_RemoveUserChatText_Response(betterproto.Message):
    pass


@dataclass
class CBroadcast_GetBroadcastChatUserNames_Request(betterproto.Message):
    chat_id: float = betterproto.fixed64_field(1)
    user_steamid: List[float] = betterproto.fixed64_field(2)


@dataclass
class CBroadcast_GetBroadcastChatUserNames_Response(betterproto.Message):
    persona_names: List["CBroadcast_GetBroadcastChatUserNames_ResponsePersonaName"] = betterproto.message_field(1)


@dataclass
class CBroadcast_GetBroadcastChatUserNames_ResponsePersonaName(betterproto.Message):
    steam_id: float = betterproto.fixed64_field(1)
    persona: str = betterproto.string_field(2)


@dataclass
class CBroadcast_StartBuildClip_Request(betterproto.Message):
    steamid: float = betterproto.fixed64_field(1)
    broadcast_session_id: float = betterproto.fixed64_field(2)
    first_segment: int = betterproto.int32_field(3)
    num_segments: int = betterproto.int32_field(4)
    clip_description: str = betterproto.string_field(5)


@dataclass
class CBroadcast_StartBuildClip_Response(betterproto.Message):
    broadcast_clip_id: float = betterproto.fixed64_field(1)


@dataclass
class CBroadcast_GetBuildClipStatus_Request(betterproto.Message):
    broadcast_clip_id: float = betterproto.fixed64_field(1)


@dataclass
class CBroadcast_GetBuildClipStatus_Response(betterproto.Message):
    pass


@dataclass
class CBroadcast_SetClipDetails_Request(betterproto.Message):
    broadcast_clip_id: int = betterproto.uint64_field(1)
    start_time: int = betterproto.uint32_field(2)
    end_time: int = betterproto.uint32_field(3)
    video_description: str = betterproto.string_field(4)


@dataclass
class CBroadcast_SetClipDetails_Response(betterproto.Message):
    pass


@dataclass
class CBroadcast_GetClipDetails_Request(betterproto.Message):
    broadcast_clip_id: int = betterproto.uint64_field(1)


@dataclass
class CBroadcast_GetClipDetails_Response(betterproto.Message):
    broadcast_clip_id: int = betterproto.uint64_field(1)
    video_id: int = betterproto.uint64_field(2)
    channel_id: int = betterproto.uint64_field(3)
    app_id: int = betterproto.uint32_field(4)
    accountid_broadcaster: int = betterproto.uint32_field(5)
    accountid_clipmaker: int = betterproto.uint32_field(6)
    video_description: str = betterproto.string_field(7)
    start_time: int = betterproto.uint32_field(8)
    length_milliseconds: int = betterproto.uint32_field(9)
    thumbnail_path: str = betterproto.string_field(10)


@dataclass
class CBroadcast_SetRTMPInfo_Request(betterproto.Message):
    broadcast_permission: int = betterproto.int32_field(1)
    update_token: bool = betterproto.bool_field(2)
    broadcast_delay: int = betterproto.int32_field(3)
    app_id: int = betterproto.uint32_field(4)
    required_app_id: int = betterproto.uint32_field(5)
    broadcast_chat_permission: "EBroadcastChatPermission" = betterproto.enum_field(6)
    broadcast_buffer: int = betterproto.int32_field(7)
    steamid: float = betterproto.fixed64_field(8)
    chat_rate_limit: int = betterproto.uint32_field(9)
    enable_replay: bool = betterproto.bool_field(10)


@dataclass
class CBroadcast_SetRTMPInfo_Response(betterproto.Message):
    pass


@dataclass
class CBroadcast_GetRTMPInfo_Request(betterproto.Message):
    ip: int = betterproto.uint32_field(1)
    steamid: float = betterproto.fixed64_field(2)


@dataclass
class CBroadcast_GetRTMPInfo_Response(betterproto.Message):
    broadcast_permission: int = betterproto.int32_field(1)
    rtmp_host: str = betterproto.string_field(2)
    rtmp_token: str = betterproto.string_field(3)
    broadcast_delay: int = betterproto.int32_field(4)
    app_id: int = betterproto.uint32_field(5)
    required_app_id: int = betterproto.uint32_field(6)
    broadcast_chat_permission: "EBroadcastChatPermission" = betterproto.enum_field(7)
    broadcast_buffer: int = betterproto.int32_field(8)
    steamid: float = betterproto.fixed64_field(9)
    chat_rate_limit: int = betterproto.uint32_field(10)
    enable_replay: bool = betterproto.bool_field(11)


@dataclass
class CBroadcast_WebRTCHaveTURNServer_Notification(betterproto.Message):
    broadcast_session_id: float = betterproto.fixed64_field(1)
    turn_server: str = betterproto.string_field(2)


@dataclass
class CBroadcast_WebRTCStartResult_Request(betterproto.Message):
    webrtc_session_id: float = betterproto.fixed64_field(1)
    started: bool = betterproto.bool_field(2)
    offer: str = betterproto.string_field(3)
    resolution_x: int = betterproto.uint32_field(4)
    resolution_y: int = betterproto.uint32_field(5)
    fps: int = betterproto.uint32_field(6)


@dataclass
class CBroadcast_WebRTCStartResult_Response(betterproto.Message):
    pass


@dataclass
class CBroadcast_WebRTCStopped_Request(betterproto.Message):
    webrtc_session_id: float = betterproto.fixed64_field(1)


@dataclass
class CBroadcast_WebRTCStopped_Response(betterproto.Message):
    pass


@dataclass
class CBroadcast_WebRTCSetAnswer_Request(betterproto.Message):
    broadcaster_steamid: float = betterproto.fixed64_field(1)
    webrtc_session_id: float = betterproto.fixed64_field(2)
    answer: str = betterproto.string_field(3)


@dataclass
class CBroadcast_WebRTCSetAnswer_Response(betterproto.Message):
    pass


@dataclass
class CBroadcast_WebRTCLookupTURNServer_Request(betterproto.Message):
    cellid: int = betterproto.uint32_field(1)


@dataclass
class CBroadcast_WebRTCLookupTURNServer_Response(betterproto.Message):
    turn_server: str = betterproto.string_field(1)


@dataclass
class CBroadcast_WebRTC_Candidate(betterproto.Message):
    sdp_mid: str = betterproto.string_field(1)
    sdp_mline_index: int = betterproto.int32_field(2)
    candidate: str = betterproto.string_field(3)


@dataclass
class CBroadcast_WebRTCAddHostCandidate_Request(betterproto.Message):
    webrtc_session_id: float = betterproto.fixed64_field(1)
    candidate: "CBroadcast_WebRTC_Candidate" = betterproto.message_field(2)


@dataclass
class CBroadcast_WebRTCAddHostCandidate_Response(betterproto.Message):
    pass


@dataclass
class CBroadcast_WebRTCAddViewerCandidate_Request(betterproto.Message):
    broadcaster_steamid: float = betterproto.fixed64_field(1)
    webrtc_session_id: float = betterproto.fixed64_field(2)
    candidate: "CBroadcast_WebRTC_Candidate" = betterproto.message_field(3)


@dataclass
class CBroadcast_WebRTCAddViewerCandidate_Response(betterproto.Message):
    pass


@dataclass
class CBroadcast_WebRTCGetHostCandidates_Request(betterproto.Message):
    broadcaster_steamid: float = betterproto.fixed64_field(1)
    webrtc_session_id: float = betterproto.fixed64_field(2)
    candidate_generation: int = betterproto.uint32_field(3)


@dataclass
class CBroadcast_WebRTCGetHostCandidates_Response(betterproto.Message):
    candidate_generation: int = betterproto.uint32_field(1)
    candidates: List["CBroadcast_WebRTC_Candidate"] = betterproto.message_field(2)


@dataclass
class CBroadcast_GetBroadcastUploadStats_Request(betterproto.Message):
    row_limit: int = betterproto.uint32_field(1)
    start_time: int = betterproto.uint32_field(2)
    upload_id: int = betterproto.uint64_field(3)
    steamid: float = betterproto.fixed64_field(4)
    session_id: int = betterproto.uint64_field(5)


@dataclass
class CBroadcast_GetBroadcastUploadStats_Response(betterproto.Message):
    upload_stats: List["CBroadcast_GetBroadcastUploadStats_ResponseUploadStats"] = betterproto.message_field(1)


@dataclass
class CBroadcast_GetBroadcastUploadStats_ResponseUploadStats(betterproto.Message):
    upload_result: int = betterproto.uint32_field(1)
    time_stopped: int = betterproto.uint32_field(2)
    seconds_uploaded: int = betterproto.uint32_field(3)
    max_viewers: int = betterproto.uint32_field(4)
    resolution_x: int = betterproto.uint32_field(5)
    resolution_y: int = betterproto.uint32_field(6)
    avg_bandwidth: int = betterproto.uint32_field(7)
    total_bytes: int = betterproto.uint64_field(8)
    app_id: int = betterproto.uint32_field(9)
    total_unique_viewers: int = betterproto.uint32_field(10)
    total_seconds_watched: int = betterproto.uint64_field(11)
    time_started: int = betterproto.uint32_field(12)
    upload_id: int = betterproto.uint64_field(13)
    local_address: str = betterproto.string_field(14)
    remote_address: str = betterproto.string_field(15)
    frames_per_second: int = betterproto.uint32_field(16)
    num_representations: int = betterproto.uint32_field(17)
    app_name: str = betterproto.string_field(18)
    is_replay: bool = betterproto.bool_field(19)
    session_id: int = betterproto.uint64_field(20)


@dataclass
class CBroadcast_GetBroadcastViewerStats_Request(betterproto.Message):
    upload_id: int = betterproto.uint64_field(1)
    steamid: float = betterproto.fixed64_field(2)


@dataclass
class CBroadcast_GetBroadcastViewerStats_Response(betterproto.Message):
    viewer_stats: List["CBroadcast_GetBroadcastViewerStats_ResponseViewerStats"] = betterproto.message_field(1)
    country_stats: List["CBroadcast_GetBroadcastViewerStats_ResponseCountryStats"] = betterproto.message_field(2)


@dataclass
class CBroadcast_GetBroadcastViewerStats_ResponseViewerStats(betterproto.Message):
    time: int = betterproto.uint32_field(1)
    num_viewers: int = betterproto.uint32_field(2)


@dataclass
class CBroadcast_GetBroadcastViewerStats_ResponseCountryStats(betterproto.Message):
    country_code: str = betterproto.string_field(1)
    num_viewers: int = betterproto.uint32_field(2)


@dataclass
class CBroadcast_BroadcastViewerState_Notification(betterproto.Message):
    steamid: float = betterproto.fixed64_field(1)
    state: "CBroadcast_BroadcastViewerState_NotificationEViewerState" = betterproto.enum_field(2)


@dataclass
class CBroadcast_WaitingBroadcastViewer_Notification(betterproto.Message):
    broadcast_id: float = betterproto.fixed64_field(1)


@dataclass
class CBroadcast_BroadcastUploadStarted_Notification(betterproto.Message):
    broadcast_id: float = betterproto.fixed64_field(1)
    upload_token: str = betterproto.string_field(2)
    upload_address: str = betterproto.string_field(3)
    http_address: str = betterproto.string_field(4)
    broadcast_upload_id: float = betterproto.fixed64_field(5)
    heartbeat_interval_seconds: int = betterproto.uint32_field(6)
    is_rtmp: bool = betterproto.bool_field(7)


@dataclass
class CBroadcast_StopBroadcastUpload_Notification(betterproto.Message):
    broadcast_id: float = betterproto.fixed64_field(1)
    broadcast_relay_id: float = betterproto.fixed64_field(2)
    upload_result: int = betterproto.uint32_field(3)
    too_many_poor_uploads: bool = betterproto.bool_field(4)


@dataclass
class CBroadcast_SessionClosed_Notification(betterproto.Message):
    broadcast_id: float = betterproto.fixed64_field(1)


@dataclass
class CBroadcast_ViewerBroadcastInvite_Notification(betterproto.Message):
    broadcaster_steamid: float = betterproto.fixed64_field(1)


@dataclass
class CBroadcast_BroadcastStatus_Notification(betterproto.Message):
    broadcast_id: float = betterproto.fixed64_field(1)
    num_viewers: int = betterproto.int32_field(2)


@dataclass
class CBroadcast_BroadcastChannelLive_Notification(betterproto.Message):
    broadcast_channel_id: float = betterproto.fixed64_field(1)
    broadcast_channel_name: str = betterproto.string_field(2)
    broadcast_channel_avatar: str = betterproto.string_field(3)


@dataclass
class CBroadcast_SendThumbnailToRelay_Notification(betterproto.Message):
    thumbnail_upload_token: str = betterproto.string_field(1)
    thumbnail_broadcast_session_id: float = betterproto.fixed64_field(2)
    thumbnail_data: bytes = betterproto.bytes_field(3)
    thumbnail_width: int = betterproto.uint32_field(4)
    thumbnail_height: int = betterproto.uint32_field(5)


@dataclass
class CBroadcast_WebRTCNeedTURNServer_Notification(betterproto.Message):
    broadcast_session_id: float = betterproto.fixed64_field(1)


@dataclass
class CBroadcast_WebRTCStart_Notification(betterproto.Message):
    broadcast_session_id: float = betterproto.fixed64_field(1)
    webrtc_session_id: float = betterproto.fixed64_field(2)
    viewer_steamid: float = betterproto.fixed64_field(3)
    viewer_token: float = betterproto.fixed64_field(4)


@dataclass
class CBroadcast_WebRTCSetAnswer_Notification(betterproto.Message):
    broadcast_session_id: float = betterproto.fixed64_field(1)
    webrtc_session_id: float = betterproto.fixed64_field(2)
    answer: str = betterproto.string_field(3)


@dataclass
class CBroadcast_WebRTCAddViewerCandidate_Notification(betterproto.Message):
    broadcast_session_id: float = betterproto.fixed64_field(1)
    webrtc_session_id: float = betterproto.fixed64_field(2)
    candidate: "CBroadcast_WebRTC_Candidate" = betterproto.message_field(3)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_credentials.proto
# plugin: python-betterproto

from dataclasses import dataclass
from typing import List

import betterproto


@dataclass
class CCredentials_TestAvailablePassword_Request(betterproto.Message):
    password: str = betterproto.string_field(1)
    sha_digest_password: bytes = betterproto.bytes_field(2)
    account_name: str = betterproto.string_field(3)


@dataclass
class CCredentials_TestAvailablePassword_Response(betterproto.Message):
    is_valid: bool = betterproto.bool_field(3)


@dataclass
class CCredentials_GetSteamGuardDetails_Request(betterproto.Message):
    include_new_authentications: bool = betterproto.bool_field(1)
    webcookie: str = betterproto.string_field(2)
    timestamp_minimum_wanted: float = betterproto.fixed32_field(3)
    ipaddress: int = betterproto.int32_field(4)


@dataclass
class CCredentials_GetSteamGuardDetails_Response(betterproto.Message):
    is_steamguard_enabled: bool = betterproto.bool_field(1)
    timestamp_steamguard_enabled: float = betterproto.fixed32_field(2)
    deprecated_newauthentication: List[
        "CCredentials_GetSteamGuardDetails_ResponseNewAuthentication"
    ] = betterproto.message_field(3)
    deprecated_machine_name_userchosen: str = betterproto.string_field(4)
    deprecated_timestamp_machine_steamguard_enabled: float = betterproto.fixed32_field(5)
    deprecated_authentication_exists_from_geoloc_before_mintime: bool = betterproto.bool_field(6)
    deprecated_machine_id: int = betterproto.uint64_field(7)
    session_data: List["CCredentials_GetSteamGuardDetails_ResponseSessionData"] = betterproto.message_field(8)
    is_twofactor_enabled: bool = betterproto.bool_field(9)
    timestamp_twofactor_enabled: float = betterproto.fixed32_field(10)
    is_phone_verified: bool = betterproto.bool_field(11)


@dataclass
class CCredentials_GetSteamGuardDetails_ResponseNewAuthentication(betterproto.Message):
    timestamp_steamguard_enabled: float = betterproto.fixed32_field(1)
    is_web_cookie: bool = betterproto.bool_field(2)
    ipaddress: int = betterproto.int32_field(3)
    geoloc_info: str = betterproto.string_field(4)
    is_remembered: bool = betterproto.bool_field(5)
    machine_name_user_supplied: str = betterproto.string_field(6)
    status: int = betterproto.int32_field(7)


@dataclass
class CCredentials_GetSteamGuardDetails_ResponseSessionData(betterproto.Message):
    machine_id: int = betterproto.uint64_field(1)
    machine_name_userchosen: str = betterproto.string_field(2)
    timestamp_machine_steamguard_enabled: float = betterproto.fixed32_field(3)
    authentication_exists_from_geoloc_before_mintime: bool = betterproto.bool_field(4)
    newauthentication: List["CCredentials_GetSteamGuardDetails_ResponseNewAuthentication"] = betterproto.message_field(
        5
    )
    authentication_exists_from_same_ip_before_mintime: bool = betterproto.bool_field(6)
    public_ipv4: int = betterproto.uint32_field(7)
    public_ip_address: str = betterproto.string_field(8)


@dataclass
class CCredentials_NewMachineNotificationDialog_Request(betterproto.Message):
    is_approved: bool = betterproto.bool_field(1)
    is_wizard_complete: bool = betterproto.bool_field(2)


@dataclass
class CCredentials_NewMachineNotificationDialog_Response(betterproto.Message):
    pass


@dataclass
class CCredentials_ValidateEmailAddress_Request(betterproto.Message):
    stoken: str = betterproto.string_field(1)


@dataclass
class CCredentials_ValidateEmailAddress_Response(betterproto.Message):
    was_validated: bool = betterproto.bool_field(1)


@dataclass
class CCredentials_SteamGuardPhishingReport_Request(betterproto.Message):
    param_string: str = betterproto.string_field(1)
    ipaddress_actual: int = betterproto.uint32_field(2)


@dataclass
class CCredentials_SteamGuardPhishingReport_Response(betterproto.Message):
    ipaddress_loginattempt: int = betterproto.uint32_field(1)
    countryname_loginattempt: str = betterproto.string_field(2)
    statename_loginattempt: str = betterproto.string_field(3)
    cityname_loginattempt: str = betterproto.string_field(4)
    ipaddress_actual: int = betterproto.uint32_field(5)
    countryname_actual: str = betterproto.string_field(6)
    statename_actual: str = betterproto.string_field(7)
    cityname_actual: str = betterproto.string_field(8)
    steamguard_code: str = betterproto.string_field(9)


@dataclass
class CCredentials_LastCredentialChangeTime_Request(betterproto.Message):
    user_changes_only: bool = betterproto.bool_field(1)


@dataclass
class CCredentials_LastCredentialChangeTime_Response(betterproto.Message):
    timestamp_last_password_change: float = betterproto.fixed32_field(1)
    timestamp_last_email_change: float = betterproto.fixed32_field(2)
    timestamp_last_password_reset: float = betterproto.fixed32_field(3)


@dataclass
class CCredentials_GetAccountAuthSecret_Request(betterproto.Message):
    pass


@dataclass
class CCredentials_GetAccountAuthSecret_Response(betterproto.Message):
    secret_id: int = betterproto.int32_field(1)
    secret: bytes = betterproto.bytes_field(2)



# -*- coding: utf-8 -*-

from . import (
    steammessages_base,
    steammessages_clientserver,
    steammessages_clientserver_2,
    steammessages_clientserver_friends,
    steammessages_clientserver_login,
)
from .emsg import *

__all__ = ("PROTOBUFS",)


PROTOBUFS = {
    EMsg.Multi: steammessages_base.CMsgMulti,
    EMsg.ClientHeartBeat: steammessages_clientserver_login.CMsgClientHeartBeat,
    EMsg.ClientServerTimestampRequest: steammessages_clientserver_login.CMsgClientServerTimestampRequest,
    EMsg.TimestampResponse: steammessages_clientserver_login.CMsgClientServerTimestampResponse,
    EMsg.ClientLogon: steammessages_clientserver_login.CMsgClientLogon,
    EMsg.ClientLogonGameServer: steammessages_clientserver_login.CMsgClientLogon,
    EMsg.ClientLogOnResponse: steammessages_clientserver_login.CMsgClientLogonResponse,
    EMsg.ClientLogOff: steammessages_clientserver_login.CMsgClientLogOff,
    EMsg.ClientLoggedOff: steammessages_clientserver_login.CMsgClientLoggedOff,
    EMsg.ClientNewLoginKey: steammessages_clientserver_login.CMsgClientNewLoginKey,
    EMsg.ClientNewLoginKeyAccepted: steammessages_clientserver_login.CMsgClientNewLoginKeyAccepted,
    EMsg.ClientRequestWebAPIAuthenticateUserNonce: steammessages_clientserver_login.CMsgClientRequestWebAPIAuthenticateUserNonce,
    EMsg.ClientRequestWebAPIAuthenticateUserNonceResponse: steammessages_clientserver_login.CMsgClientRequestWebAPIAuthenticateUserNonceResponse,
    EMsg.ClientCMList: steammessages_clientserver.CMsgClientCMList,
    EMsg.ClientItemAnnouncements: steammessages_clientserver_2.CMsgClientItemAnnouncements,
    EMsg.ClientRequestItemAnnouncements: steammessages_clientserver_2.CMsgClientRequestItemAnnouncements,
    EMsg.ClientCommentNotifications: steammessages_clientserver_2.CMsgClientCommentNotifications,
    EMsg.ClientRequestCommentNotifications: steammessages_clientserver_2.CMsgClientRequestCommentNotifications,
    EMsg.ClientUserNotifications: steammessages_clientserver_2.CMsgClientUserNotifications,
    EMsg.ClientChatOfflineMessageNotification: steammessages_clientserver_2.CMsgClientOfflineMessageNotification,
    EMsg.ClientChatRequestOfflineMessageCount: steammessages_clientserver_2.CMsgClientRequestOfflineMessageCount,
    EMsg.ClientGamesPlayed: steammessages_clientserver.CMsgClientGamesPlayed,
    EMsg.ClientGamesPlayedWithDataBlob: steammessages_clientserver.CMsgClientGamesPlayed,
    EMsg.ClientAccountInfo: steammessages_clientserver_login.CMsgClientAccountInfo,
    EMsg.ClientEmailAddrInfo: steammessages_clientserver_2.CMsgClientEmailAddrInfo,
    EMsg.ClientIsLimitedAccount: steammessages_clientserver.CMsgClientIsLimitedAccount,
    EMsg.ClientWalletInfoUpdate: steammessages_clientserver.CMsgClientWalletInfoUpdate,
    EMsg.ClientLicenseList: steammessages_clientserver.CMsgClientLicenseList,
    EMsg.ClientGMSServerQuery: steammessages_clientserver_2.CMsgClientGMSServerQuery,
    EMsg.GMSClientServerQueryResponse: steammessages_clientserver_2.CMsgGMSClientServerQueryResponse,
    EMsg.ClientPICSChangesSinceRequest: steammessages_clientserver.CMsgClientPICSChangesSinceRequest,
    EMsg.ClientPICSChangesSinceResponse: steammessages_clientserver.CMsgClientPICSChangesSinceResponse,
    EMsg.ClientPICSProductInfoRequest: steammessages_clientserver.CMsgClientPICSProductInfoRequest,
    EMsg.ClientPICSProductInfoResponse: steammessages_clientserver.CMsgClientPICSProductInfoResponse,
    EMsg.ClientPICSAccessTokenRequest: steammessages_clientserver.CMsgClientPICSAccessTokenRequest,
    EMsg.ClientPICSAccessTokenResponse: steammessages_clientserver.CMsgClientPICSAccessTokenResponse,
    EMsg.EconTrading_InitiateTradeRequest: steammessages_clientserver_2.CMsgTrading_InitiateTradeRequest,
    EMsg.EconTrading_InitiateTradeResponse: steammessages_clientserver_2.CMsgTrading_InitiateTradeResponse,
    EMsg.EconTrading_CancelTradeRequest: steammessages_clientserver_2.CMsgTrading_CancelTradeRequest,
    EMsg.EconTrading_InitiateTradeProposed: steammessages_clientserver_2.CMsgTrading_InitiateTradeRequest,
    EMsg.EconTrading_InitiateTradeResult: steammessages_clientserver_2.CMsgTrading_InitiateTradeResponse,
    EMsg.EconTrading_StartSession: steammessages_clientserver_2.CMsgTrading_StartSession,
    EMsg.ClientChangeStatus: steammessages_clientserver_friends.CMsgClientChangeStatus,
    EMsg.ClientAddFriend: steammessages_clientserver_friends.CMsgClientAddFriend,
    EMsg.ClientAddFriendResponse: steammessages_clientserver_friends.CMsgClientAddFriendResponse,
    EMsg.ClientRemoveFriend: steammessages_clientserver_friends.CMsgClientRemoveFriend,
    EMsg.ClientFSGetFriendsSteamLevels: steammessages_clientserver_2.CMsgClientFSGetFriendsSteamLevels,
    EMsg.ClientFSGetFriendsSteamLevelsResponse: steammessages_clientserver_2.CMsgClientFSGetFriendsSteamLevelsResponse,
    EMsg.ClientPersonaState: steammessages_clientserver_friends.CMsgClientPersonaState,
    EMsg.ClientClanState: steammessages_clientserver.CMsgClientClanState,
    EMsg.ClientFriendsList: steammessages_clientserver_friends.CMsgClientFriendsList,
    EMsg.ClientRequestFriendData: steammessages_clientserver_friends.CMsgClientRequestFriendData,
    EMsg.ClientFriendMsg: steammessages_clientserver_friends.CMsgClientFriendMsg,
    EMsg.ClientChatInvite: steammessages_clientserver.CMsgClientChatInvite,
    EMsg.ClientFriendMsgIncoming: steammessages_clientserver_friends.CMsgClientFriendMsgIncoming,
    EMsg.ClientFriendMsgEchoToSender: steammessages_clientserver_friends.CMsgClientFriendMsgIncoming,
    EMsg.ClientChatGetFriendMessageHistory: steammessages_clientserver_2.CMsgClientChatGetFriendMessageHistory,
    EMsg.ClientChatGetFriendMessageHistoryResponse: steammessages_clientserver_2.CMsgClientChatGetFriendMessageHistoryResponse,
    EMsg.ClientFriendsGroupsList: steammessages_clientserver_friends.CMsgClientFriendsGroupsList,
    EMsg.AMClientCreateFriendsGroup: steammessages_clientserver_friends.CMsgClientCreateFriendsGroup,
    EMsg.AMClientCreateFriendsGroupResponse: steammessages_clientserver_friends.CMsgClientCreateFriendsGroupResponse,
    EMsg.AMClientDeleteFriendsGroup: steammessages_clientserver_friends.CMsgClientDeleteFriendsGroup,
    EMsg.AMClientDeleteFriendsGroupResponse: steammessages_clientserver_friends.CMsgClientDeleteFriendsGroupResponse,
    EMsg.AMClientAddFriendToGroup: steammessages_clientserver_friends.CMsgClientAddFriendToGroup,
    EMsg.AMClientAddFriendToGroupResponse: steammessages_clientserver_friends.CMsgClientAddFriendToGroupResponse,
    EMsg.AMClientRemoveFriendFromGroup: steammessages_clientserver_friends.CMsgClientRemoveFriendFromGroup,
    EMsg.AMClientRemoveFriendFromGroupResponse: steammessages_clientserver_friends.CMsgClientRemoveFriendFromGroupResponse,
    EMsg.ClientPlayerNicknameList: steammessages_clientserver_friends.CMsgClientPlayerNicknameList,
    EMsg.AMClientSetPlayerNickname: steammessages_clientserver_friends.CMsgClientSetPlayerNickname,
    EMsg.AMClientSetPlayerNicknameResponse: steammessages_clientserver_friends.CMsgClientSetPlayerNicknameResponse,
    EMsg.ClientRegisterKey: steammessages_clientserver_2.CMsgClientRegisterKey,
    EMsg.ClientPurchaseResponse: steammessages_clientserver_2.CMsgClientPurchaseResponse,
    EMsg.ClientRequestFreeLicense: steammessages_clientserver_2.CMsgClientRequestFreeLicense,
    EMsg.ClientRequestFreeLicenseResponse: steammessages_clientserver_2.CMsgClientRequestFreeLicenseResponse,
    EMsg.ClientGetNumberOfCurrentPlayersDP: steammessages_clientserver_2.CMsgDPGetNumberOfCurrentPlayers,
    EMsg.ClientGetNumberOfCurrentPlayersDPResponse: steammessages_clientserver_2.CMsgDPGetNumberOfCurrentPlayersResponse,
    EMsg.ClientGetAppOwnershipTicket: steammessages_clientserver.CMsgClientGetAppOwnershipTicket,
    EMsg.ClientGetAppOwnershipTicketResponse: steammessages_clientserver.CMsgClientGetAppOwnershipTicketResponse,
    EMsg.ClientGameConnectTokens: steammessages_clientserver.CMsgClientGameConnectTokens,
    EMsg.ClientAuthList: steammessages_clientserver.CMsgClientAuthList,
    EMsg.ClientAuthListAck: steammessages_clientserver.CMsgClientAuthListAck,
    EMsg.ClientTicketAuthComplete: steammessages_clientserver.CMsgClientTicketAuthComplete,
    EMsg.ClientCurrentUIMode: steammessages_clientserver_2.CMsgClientUIMode,
    EMsg.ClientVanityURLChangedNotification: steammessages_clientserver_2.CMsgClientVanityURLChangedNotification,
    EMsg.ClientAMGetPersonaNameHistory: steammessages_clientserver.CMsgClientAMGetPersonaNameHistory,
    EMsg.ClientAMGetPersonaNameHistoryResponse: steammessages_clientserver.CMsgClientAMGetPersonaNameHistoryResponse,
    EMsg.ClientUnlockStreaming: steammessages_clientserver_2.CMsgAMUnlockStreaming,
    EMsg.ClientUnlockStreamingResponse: steammessages_clientserver_2.CMsgAMUnlockStreamingResponse,
    EMsg.ClientGetDepotDecryptionKey: steammessages_clientserver_2.CMsgClientGetDepotDecryptionKey,
    EMsg.ClientGetDepotDecryptionKeyResponse: steammessages_clientserver_2.CMsgClientGetDepotDecryptionKeyResponse,
    EMsg.ClientGetCDNAuthToken: steammessages_clientserver_2.CMsgClientGetCDNAuthToken,
    EMsg.ClientGetCDNAuthTokenResponse: steammessages_clientserver_2.CMsgClientGetCDNAuthTokenResponse,
    EMsg.ClientCheckAppBetaPassword: steammessages_clientserver_2.CMsgClientCheckAppBetaPassword,
    EMsg.ClientCheckAppBetaPasswordResponse: steammessages_clientserver_2.CMsgClientCheckAppBetaPasswordResponse,
    EMsg.ClientKickPlayingSession: steammessages_clientserver_2.CMsgClientKickPlayingSession,
    EMsg.ClientConcurrentSessionsBase: steammessages_clientserver_2.CMsgClientPlayingSessionState,
    EMsg.ClientToGC: steammessages_clientserver_2.CMsgGCClient,
    EMsg.ClientFromGC: steammessages_clientserver_2.CMsgGCClient,
    EMsg.ClientRichPresenceUpload: steammessages_clientserver_2.CMsgClientRichPresenceUpload,
    EMsg.ClientGetEmoticonList: steammessages_clientserver_friends.CMsgClientGetEmoticonList,
    EMsg.ClientEmoticonList: steammessages_clientserver_friends.CMsgClientEmoticonList,
    EMsg.ClientServersAvailable: steammessages_clientserver.CMsgClientServersAvailable,
    EMsg.ClientRequestedClientStats: steammessages_clientserver.CMsgClientRequestedClientStats,
}



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: gc.proto
# plugin: python-betterproto

from dataclasses import dataclass

import betterproto


class GCProtoBufMsgSrc(betterproto.Enum):
    Unspecified = 0
    FromSystem = 1
    FromSteamID = 2
    FromGC = 3
    ReplySystem = 4


@dataclass
class CMsgProtoBufHeader(betterproto.Message):
    client_steam_id: float = betterproto.fixed64_field(1)
    client_session_id: int = betterproto.int32_field(2)
    source_app_id: int = betterproto.uint32_field(3)
    job_id_source: float = betterproto.fixed64_field(10)
    job_id_target: float = betterproto.fixed64_field(11)
    target_job_name: str = betterproto.string_field(12)
    eresult: int = betterproto.int32_field(13)
    error_message: str = betterproto.string_field(14)
    gc_msg_src: "GCProtoBufMsgSrc" = betterproto.enum_field(200)
    gc_dir_index_source: int = betterproto.uint32_field(201)



# -*- coding: utf-8 -*-

"""
The MIT License (MIT)

Copyright (c) 2020 James

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import time as _time
from typing import TYPE_CHECKING, List, Tuple

from ...enums import IntEnum as IntEnumBase
from .errors import CommandOnCooldown

if TYPE_CHECKING:
    from enum import IntEnum

    from .context import Context


__all__ = (
    "BucketType",
    "Cooldown",
)


class BucketType(IntEnumBase):
    User: "IntEnum" = 1
    Member: "IntEnum" = 2
    Group: "IntEnum" = 3
    Clan: "IntEnum" = 4
    Role: "IntEnum" = 5
    Channel: "IntEnum" = 6
    Officer: "IntEnum" = 7

    def get_bucket(self, ctx: "Context"):
        if self == self.User:
            return ctx.author.id
        elif self is self.Member:
            return (ctx.clan and ctx.clan.id), ctx.author.id
        elif self == self.Group:
            return (ctx.group or ctx.author).id
        elif self is self.Role:
            return
        elif self == self.Clan:
            return (ctx.clan or ctx.author).id
        elif self == self.Channel:
            return (ctx.channel or ctx.author).id
        elif self is self.Officer:
            return


class Cooldown:
    def __init__(self, rate: int, per: float, bucket: BucketType):
        self._rate = rate
        self._per = per
        self.bucket = bucket
        self._last_update = 0
        self._last_called_by: List[Tuple[int, int]] = []

    def reset(self):
        self._last_update = 0
        self._last_called_by = []

    def __call__(self, bucket):
        now = _time.time()

        for (_, time) in self._last_called_by:
            if now >= time + self._per:
                self._last_called_by.pop(0)  # FIFO makes this fine

        if self._last_update + self._per >= now:
            if len(self._last_called_by) >= self._rate:
                if bucket in [b for b, n in self._last_called_by]:
                    retry_after = now - self._last_update + self._per
                    raise CommandOnCooldown(retry_after)

        self._last_called_by.append((bucket, now))
        self._last_update = _time.time()



# -*- coding: utf-8 -*-

"""
The MIT License (MIT)

Copyright (c) 2020 James

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import asyncio
from datetime import datetime
from typing import TYPE_CHECKING, Iterable, List, Optional, Union

from .enums import ETradeOfferState
from .errors import ClientException, ConfirmationError
from .game import Game

if TYPE_CHECKING:
    from .state import ConnectionState
    from .user import BaseUser, User


__all__ = (
    "Item",
    "Asset",
    "Inventory",
    "TradeOffer",
)


class Asset:
    """A striped down version of an item.

    .. container:: operations

        .. describe:: x == y

            Checks if two assets are equal.

        .. describe:: x != y

            Checks if two assets are not equal.

    Attributes
    -------------
    game: :class:`~steam.Game`
        The game the item is from.
    asset_id: :class:`str`
        The assetid of the item.
    app_id: :class:`str`
        The appid of the item.
    amount: :class:`int`
        The amount of the same asset there are in the inventory.
    instance_id: :class:`str`
        The instanceid of the item.
    class_id: :class:`str`
        The classid of the item.
    """

    __slots__ = ("game", "amount", "app_id", "class_id", "asset_id", "instance_id")

    def __init__(self, data: dict):
        self.asset_id = int(data["assetid"])
        self.game = Game(app_id=data["appid"])
        self.app_id = self.game.app_id
        self.amount = int(data["amount"])
        self.instance_id = int(data["instanceid"])
        self.class_id = int(data["classid"])

    def __repr__(self):
        attrs = (
            "game",
            "amount",
            "class_id",
            "asset_id",
            "instance_id",
        )
        resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
        return f"<Asset {' '.join(resolved)}>"

    def __eq__(self, other):
        return isinstance(other, Asset) and self.instance_id == other.instance_id and self.class_id == other.class_id

    def to_dict(self) -> dict:
        return {
            "assetid": str(self.asset_id),
            "amount": self.amount,
            "appid": str(self.app_id),
            "contextid": str(self.game.context_id),
        }


class Item(Asset):
    """Represents an item in an User's inventory.

    .. container:: operations

        .. describe:: x == y

            Checks if two items are equal.

        .. describe:: x != y

            Checks if two items are not equal.

    Attributes
    -------------
    name: Optional[:class:`str`]
        The market_name of the item.
    display_name: Optional[:class:`str`]
        The displayed name of the item. This could be different to
        :attr:`Item.name` if the item is user re-nameable.
    colour: Optional[:class:`int`]
        The colour of the item.
    descriptions: Optional[:class:`str`]
        The descriptions of the item.
    type: Optional[:class:`str`]
        The type of the item.
    tags: Optional[:class:`str`]
        The tags of the item.
    icon_url: Optional[:class:`str`]
        The icon_url of the item. Uses the large (184x184 px) image url.
    """

    __slots__ = (
        "name",
        "type",
        "tags",
        "colour",
        "icon_url",
        "display_name",
        "descriptions",
        "_state",
        "_is_tradable",
        "_is_marketable",
    )

    def __init__(self, state: "ConnectionState", data: dict):
        super().__init__(data)
        self._state = state
        self._from_data(data)

    def __repr__(self):
        asset_repr = super().__repr__()[7:-1]
        attrs = ("name",)
        resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
        resolved.append(asset_repr)
        return f"<Item {' '.join(resolved)}>"

    def _from_data(self, data) -> None:
        self.name = data.get("market_name")
        self.display_name = data.get("name")
        self.colour = int(data["name_color"], 16) if "name_color" in data else None
        self.descriptions = data.get("descriptions")
        self.type = data.get("type")
        self.tags = data.get("tags")
        self.icon_url = (
            f'https://steamcommunity-a.akamaihd.net/economy/image/{data["icon_url_large"]}'
            if "icon_url_large" in data
            else None
        )
        self._is_tradable = bool(data.get("tradable", False))
        self._is_marketable = bool(data.get("marketable", False))

    def is_tradable(self) -> bool:
        """:class:`bool`: Whether the item is tradable."""
        return self._is_tradable

    def is_marketable(self) -> bool:
        """:class:`bool`: Whether the item is marketable."""
        return self._is_marketable


class Inventory:
    """Represents a User's inventory.

    .. container:: operations

        .. describe:: len(x)

            Returns how many items are in the inventory.

        .. describe:: iter(x)

            Iterates over the inventory's items.

        .. describe:: y in x

            Determines if an item is in the inventory based off of
            its class_id and instance_id


    Attributes
    -------------
    items: List[Union[:class:`Item`, :class:`Asset`]
        A list of the inventory's items.
    owner: :class:`~steam.User`
        The owner of the inventory.
    game: :class:`steam.Game`
        The game the inventory the game belongs to.
    """

    __slots__ = ("game", "items", "owner", "_state", "_total_inventory_count")

    def __init__(self, state: "ConnectionState", data: dict, owner: "BaseUser"):
        self._state = state
        self.owner = owner
        self.items = []
        self._update(data)

    def __repr__(self):
        attrs = ("owner", "game")
        resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
        return f"<Inventory {' '.join(resolved)}>"

    def __len__(self):
        return self._total_inventory_count

    def __iter__(self) -> Iterable[Item]:
        return (item for item in self.items)

    def __contains__(self, item):
        if isinstance(item, Asset):
            return item in self.items
        return NotImplemented

    def _update(self, data) -> None:
        try:
            self.game = Game(app_id=int(data["assets"][0]["appid"]))
        except KeyError:  # they don't have an inventory in this game
            self.game = None
            self.items = []
            self._total_inventory_count = 0
        else:
            for asset in data["assets"]:
                found = False
                for item in data["descriptions"]:
                    if item["instanceid"] == asset["instanceid"] and item["classid"] == asset["classid"]:
                        item.update(asset)
                        self.items.append(Item(state=self._state, data=item))
                        found = True
                if not found:
                    self.items.append(Asset(data=asset))
            self._total_inventory_count = data["total_inventory_count"]

    async def update(self) -> "Inventory":
        """|coro|
        Re-fetches the :class:`~steam.User`'s inventory.

        Returns
        ---------
        :class:`~steam.Inventory`
            The refreshed inventory.
        """
        data = await self._state.http.get_user_inventory(self.owner.id64, self.game.app_id, self.game.context_id)
        self._update(data)
        return self

    def filter_items(self, item_name: str, *, limit: int = None) -> List[Item]:
        """Filters items by name into a list of one type of item.

        Parameters
        ------------
        item_name: :class:`str`
            The item's name to filter for.
        limit: Optional[:class:`int`]
            The maximum amount of items to filter.

        Returns
        ---------
        List[:class:`Item`]
            List of :class:`Item`.
            Could be an empty if no matching items are found.
            This also removes the item from the inventory, if possible.
        """
        items = [item for item in self if item.name == item_name]
        items = items if limit is None else items[:limit]
        for item in items:
            self.items.remove(item)
        return items

    def get_item(self, item_name: str) -> Optional[Item]:
        """Get an item by name from a :class:`Inventory`.

        Parameters
        ----------
        item_name: :class:`str`
            The item to get from the inventory.

        Returns
        -------
        Optional[:class:`Item`]
            Returns the first found item with a matching name.
            Can be ``None`` if no matching item is found.
            This also removes the item from the inventory, if possible.
        """
        item = [item for item in self if item.name == item_name]
        if item:
            self.items.remove(item[0])
            return item[0]
        return None


class TradeOffer:
    """Represents a trade offer from/to send to a User.
    This can also be used in :meth:`steam.User.send`.

    Parameters
    ----------
    item_to_send: Optional[Union[:class:`steam.Item`, :class:`steam.Asset`]]
        The item to send with the trade offer.
    item_to_receive: Optional[Union[:class:`steam.Item`, :class:`steam.Asset`]]
        The item to receive with the trade offer.
    items_to_send: Optional[List[Union[:class:`steam.Item`, :class:`steam.Asset`]]]
        The items you are sending to the other user.
    items_to_receive: Optional[List[Union[:class:`steam.Item`, :class:`steam.Asset`]]]
        The items you are sending to the other user.
    token: Optional[:class:`str`]
        The the trade token used to send trades to users who aren't
        on the ClientUser's friend's list.
    message: Optional[:class:`str`]
         The offer message to send with the trade.

    Attributes
    -------------
    partner: Union[:class:`~steam.User`, :class:`~steam.SteamID`]
        The trade offer partner. This should only ever be a :class:`~steam.SteamID`
        if the partner's profile is private.
    items_to_send: Union[List[:class:`Item`], List[:class:`Asset`]]
        A list of items to send to the partner.
    items_to_receive: Union[List[:class:`Item`], List[:class:`Asset`]]
        A list of items to receive from the partner.
    state: :class:`~steam.ETradeOfferState`
        The offer state of the trade for the possible types see
        :class:`~steam.ETradeOfferState`.
    message: :class:`str`
        The message included with the trade offer.
    id: :class:`int`
        The trade offer id of the trade.
    expires: :class:`datetime.datetime`
        The time at which the trade automatically expires.
    escrow: Optional[:class:`datetime.datetime`]
        The time at which the escrow will end. Can be None
        if there is no escrow on the trade.
    """

    __slots__ = (
        "id",
        "state",
        "escrow",
        "partner",
        "message",
        "token",
        "expires",
        "items_to_send",
        "items_to_receive",
        "_has_been_sent",
        "_state",
        "_is_our_offer",
    )

    def __init__(
        self,
        *,
        message: str = None,
        token: str = None,
        item_to_send: Union[Item, Asset] = None,
        item_to_receive: Union[Item, Asset] = None,
        items_to_send: List[Union[Item, Asset]] = None,
        items_to_receive: List[Union[Item, Asset]] = None,
    ):
        self.items_to_receive = items_to_receive if items_to_receive else []
        self.items_to_send = items_to_send if items_to_send else []
        if item_to_receive:
            self.items_to_receive.append(item_to_receive)
        if item_to_send:
            self.items_to_send.append(item_to_send)
        self.message = message if message is not None else ""
        self.token = token
        self._has_been_sent = False
        self.partner: Optional["User"] = None
        self.state = ETradeOfferState.Invalid

    @classmethod
    async def _from_api(cls, state: "ConnectionState", data: dict) -> "TradeOffer":
        from .abc import SteamID

        trade = cls()
        trade._has_been_sent = True
        trade._state = state
        trade._update(data)
        trade.partner = await state.client.fetch_user(data["accountid_other"]) or SteamID(
            data["accountid_other"]
        )  # the account is private :(
        return trade

    def __repr__(self):
        attrs = ("id", "state", "partner")
        resolved = [f"{attr}={getattr(self, attr, None)!r}" for attr in attrs]
        return f"<TradeOffer {' '.join(resolved)}>"

    def _update(self, data) -> None:
        self.message = data.get("message") or None
        self.id = int(data["tradeofferid"])
        expires = data.get("expiration_time")
        escrow = data.get("escrow_end_date")
        self.expires = datetime.utcfromtimestamp(expires) if expires else None
        self.escrow = datetime.utcfromtimestamp(escrow) if escrow else None
        self.state = ETradeOfferState(data.get("trade_offer_state", 1))
        self.items_to_send = [Item(state=self._state, data=item) for item in data.get("items_to_give", [])]
        self.items_to_receive = [Item(state=self._state, data=item) for item in data.get("items_to_receive", [])]
        self._is_our_offer = data.get("is_our_offer", False)

    def __eq__(self, other):
        if isinstance(other, TradeOffer):
            if self._has_been_sent and other._has_been_sent:
                return self.id == other.id
        return False

    async def confirm(self) -> None:
        """|coro|
        Confirms the :class:`TradeOffer`.
        This rarely needs to be called as the client handles most of these.

        Raises
        ------
        :exc:`~steam.ClientException`
            The trade is not active.
        :exc:`~steam.ConfirmationError`
            No matching confirmation could not be found.
        """
        self._check_active()
        if self.is_gift():
            return  # no point trying to confirm it
        if not await self._state.get_and_confirm_confirmation(self.id):
            raise ConfirmationError("No matching confirmation could be found for this trade")
        del self._state._confirmations[self.id]

    async def accept(self) -> None:
        """|coro|
        Accepts the :class:`TradeOffer`.

        .. note::
            This also calls :meth:`confirm` (if necessary) so you don't have to.

        Raises
        ------
        :exc:`~steam.ClientException`
            The trade is either not active, already accepted or not from the ClientUser.
        :exc:`~steam.ConfirmationError`
            No matching confirmation could not be found.
        """
        self._check_active()
        if self.state == ETradeOfferState.Accepted:
            raise ClientException("This trade has already been accepted")
        if self.is_our_offer():
            raise ClientException("You cannot accept an offer the ClientUser has made")
        resp = await self._state.http.accept_user_trade(self.partner.id64, self.id)
        if resp.get("needs_mobile_confirmation", False):
            for tries in range(5):
                try:
                    await self.confirm()
                except ConfirmationError:
                    await asyncio.sleep(tries * 2)
                    continue

    async def decline(self) -> None:
        """|coro|
        Declines the :class:`TradeOffer`.

        Raises
        ------
        :exc:`~steam.ClientException`
            The trade is either not active, already declined or not from the ClientUser.
        """
        self._check_active()
        if self.state == ETradeOfferState.Declined:
            raise ClientException("This trade has already been declined")
        if self.is_our_offer():
            raise ClientException("You cannot decline an offer the ClientUser has made")
        await self._state.http.decline_user_trade(self.id)

    async def cancel(self) -> None:
        """|coro|
        Cancels the :class:`TradeOffer`

        Raises
        ------
        :exc:`~steam.ClientException`
            The trade is either not active, already cancelled or is from the ClientUser.
        """
        self._check_active()
        if self.state == ETradeOfferState.Canceled:
            raise ClientException("This trade has already been cancelled")
        if not self.is_gift():
            raise ClientException("Offer wasn't created by the ClientUser and therefore cannot be canceled")
        await self._state.http.cancel_user_trade(self.id)

    async def counter(self, trade: "TradeOffer") -> None:
        """|coro|
        Counters a trade offer from an :class:`User`.

        Parameters
        -----------
        trade: :class:`TradeOffer`
            The trade offer to counter with.

        Raises
        ------
        :exc:`~steam.ClientException`
            The trade from the ClientUser or it isn't active.
        """
        if not self._has_been_sent:
            raise ClientException("This trade isn't active")
        if self.is_our_offer():
            raise ClientException("You cannot counter an offer the ClientUser has made")

        to_send = [item.to_dict() for item in trade.items_to_send]
        to_receive = [item.to_dict() for item in trade.items_to_receive]
        resp = await self._state.http.send_counter_trade_offer(
            self.id, self.partner.id64, self.partner.id, to_send, to_receive, trade.token, trade.message,
        )
        if resp.get("needs_mobile_confirmation", False):
            await self._state.get_and_confirm_confirmation(int(resp["tradeofferid"]))

    def is_gift(self) -> bool:
        """:class:`bool`: Checks if an offer is a gift to the ClientUser"""
        return self.items_to_receive and not self.items_to_send

    def is_our_offer(self) -> bool:
        """:class:`bool`: Whether the offer was created by the ClientUser."""
        return self._is_our_offer

    def _check_active(self) -> None:
        if self.state not in (ETradeOfferState.Active, ETradeOfferState.ConfirmationNeed) or not self._has_been_sent:
            raise ClientException("This trade is not active")



# -*- coding: utf-8 -*-

"""
The MIT License (MIT)

Copyright (c) 2020 James

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import asyncio
import re
from datetime import datetime
from typing import (
    TYPE_CHECKING,
    AsyncIterator as _AsyncIterator,
    Awaitable,
    Callable,
    Generic,
    List,
    Optional,
    T,
    Union,
)

from bs4 import BeautifulSoup

from . import utils
from .comment import Comment
from .enums import ETradeOfferState

if TYPE_CHECKING:
    from .abc import BaseUser
    from .clan import Clan
    from .state import ConnectionState
    from .trade import TradeOffer

MaybeCoro = Callable[..., Union[bool, Awaitable[bool]]]


class AsyncIterator(_AsyncIterator, Generic[T]):
    """A class from which async iterators (see :pep:`525`) can ben easily derived.

    .. container:: operations

        .. describe:: async for x in y

            Iterates over the contents of the async iterator.

    Attributes
    ----------
    before: :class:`datetime.datetime`
        When to find objects before.
    after: :class:`datetime.datetime`
        When to find objects after.
    limit: Optional[:class:`int`]
        The maximum size of the :attr:`AsyncIterator.queue`.
    queue: :class:`asyncio.Queue`
        The queue containing the elements of the iterator.
    """

    __slots__ = ("before", "after", "limit", "queue", "_is_filled", "_state")

    def __init__(
        self, state: "ConnectionState", limit: Optional[int], before: Optional[datetime], after: Optional[datetime],
    ):
        self._state = state
        self.before = before or datetime.utcnow()
        self.after = after or datetime.utcfromtimestamp(0)
        self._is_filled = False
        self.queue: "asyncio.Queue[T]" = asyncio.Queue(maxsize=limit or 0)
        self.limit = limit

    def get(self, **attrs) -> Awaitable[Optional[T]]:
        r"""|coro|
        A helper function which is similar to :func:`~steam.utils.get`
        except it runs over the :class:`AsyncIterator`.

        This is roughly equipment to: ::

            elements = await AsyncIterator.flatten()
            element = steam.utils.get(name='Item', elements)

        Example
        -------
        Getting the last comment from a user named 'Dave' or None: ::

            msg = await User.comments().get(author__name='Dave')

        Parameters
        ----------
        \*\*attrs
            Keyword arguments that denote attributes to match.

        Returns
        -------
        Optional[Any]
            The first element from the ``iterable``
            which matches all the traits passed in ``attrs``
            or ``None`` if no matching element was found.
        """

        def predicate(elem):
            for attr, val in attrs.items():
                nested = attr.split("__")
                obj = elem
                for attribute in nested:
                    obj = getattr(obj, attribute)

                if obj != val:
                    return False
            return True

        return self.find(predicate)

    async def find(self, predicate: MaybeCoro) -> Optional[T]:
        """|coro|
        A helper function which is similar to :func:`~steam.utils.find`
        except it runs over the :class:`AsyncIterator`. However unlike
        :func:`~steam.utils.find`, the predicate provided can be a |coroutine_link|_.

        This is roughly equivalent to: ::

            elements = await AsyncIterator.flatten()
            element = steam.utils.find(elements, lambda e: e.name == 'Item')

        Example
        -------
        Getting the last trade with a message or None: ::

            def predicate(trade):
                return trade.message is not None

            trade = await Client.trade_history().find(predicate)

        Parameters
        ----------
        predicate: Callable[..., Union[:class:`bool`, Awaitable[:class:`bool`]]]
            A callable/coroutine that returns a boolean.

        Returns
        -------
        Optional[Any]
            The first element from the ``iterable``
            for which the ``predicate`` returns ``True``
            or ``None`` if no matching element was found.
        """
        while 1:
            try:
                elem = await self.next()
            except StopAsyncIteration:
                return None

            ret = await utils.maybe_coroutine(predicate, elem)
            if ret:
                return elem

    async def flatten(self) -> List[T]:
        """|coro|
        A helper function that iterates over the :class:`AsyncIterator`
        returning a list of all the elements in the iterator.

        This is equivalent to: ::

            elements = [element async for element in AsyncIterator]

        Returns
        -------
        List[Any]
            A list of every element in the iterator.
        """
        return [element async for element in self]

    async def __anext__(self) -> T:
        return await self.next()

    async def next(self) -> T:
        """|coro|
        Advances the iterator by one, if possible.

        Raises
        --------
        :exc:`StopAsyncIteration`:
            There are no more elements in the iterator.
        """
        if self.queue.empty():
            if self._is_filled:
                raise StopAsyncIteration
            await self.fill()
            if self.queue.empty():  # yikes
                raise StopAsyncIteration
            self._is_filled = True
        return self.queue.get_nowait()

    async def fill(self) -> None:
        pass


class CommentsIterator(AsyncIterator["Comment"]):
    __slots__ = ("owner",)

    def __init__(
        self, state: "ConnectionState", before: datetime, after: datetime, limit: int, owner: Union["BaseUser", "Clan"],
    ):
        super().__init__(state, limit, before, after)
        self.owner = owner

    async def fill(self) -> None:
        from .user import BaseUser, User

        data = await self._state.http.get_comments(
            id64=self.owner.id64,
            limit=self.limit,
            comment_type="Profile" if isinstance(self.owner, BaseUser) else "Clan",
        )
        soup = BeautifulSoup(data["comments_html"], "html.parser")
        comments = soup.find_all("div", attrs={"class": "commentthread_comment responsive_body_text"})
        to_fetch = []

        for comment in comments:
            timestamp = comment.find("span", attrs={"class": "commentthread_comment_timestamp"})["data-timestamp"]
            timestamp = datetime.utcfromtimestamp(int(timestamp))
            if self.after < timestamp < self.before:
                author_id = int(comment.find("a", attrs={"class": "commentthread_author_link"})["data-miniprofile"])
                comment_id = int(re.findall(r"comment_([0-9]*)", str(comment))[0])
                content = comment.find("div", attrs={"class": "commentthread_comment_text"}).get_text().strip()
                to_fetch.append(utils.make_steam64(author_id))
                comment = Comment(
                    state=self._state,
                    id=comment_id,
                    timestamp=timestamp,
                    content=content,
                    author=author_id,
                    owner=self.owner,
                )
                try:
                    self.queue.put_nowait(comment)
                except asyncio.QueueFull:
                    return
        users = await self._state.http.get_users(to_fetch)
        for user in users:
            author = User(state=self._state, data=user)
            for comment in self.queue._queue:
                if comment.author == author.id:
                    comment.author = author


class TradesIterator(AsyncIterator["TradeOffer"]):
    __slots__ = ("_active_only",)

    def __init__(
        self, state: "ConnectionState", limit: int, before: datetime, after: datetime, active_only: bool,
    ):
        super().__init__(state, limit, before, after)
        self._active_only = active_only

    async def fill(self) -> None:
        resp = await self._state.http.get_trade_history(100, None)
        resp = resp["response"]
        total = resp.get("total_trades", 0)
        if not total:
            return

        descriptions = resp.get("descriptions", [])
        try:
            from .trade import TradeOffer

            async def process_trade(data: dict, descriptions: dict):
                if self.after.timestamp() < data["time_init"] < self.before.timestamp():
                    for item in descriptions:
                        for asset in data.get("assets_received", []):
                            if item["classid"] == asset["classid"] and item["instanceid"] == asset["instanceid"]:
                                asset.update(item)
                        for asset in data.get("assets_given", []):
                            if item["classid"] == asset["classid"] and item["instanceid"] == asset["instanceid"]:
                                asset.update(item)

                    # patch in the attributes cause steam is cool
                    data["tradeofferid"] = data["tradeid"]
                    data["accountid_other"] = data["steamid_other"]
                    data["trade_offer_state"] = data["status"]
                    data["items_to_give"] = data.get("assets_given", [])
                    data["items_to_receive"] = data.get("assets_received", [])

                    trade = await TradeOffer._from_api(state=self._state, data=data)

                    try:
                        if not self._active_only:
                            self.queue.put_nowait(trade)
                        elif self._active_only and trade.state in (
                            ETradeOfferState.Active,
                            ETradeOfferState.ConfirmationNeed,
                        ):
                            self.queue.put_nowait(trade)

                    except asyncio.QueueFull:
                        raise StopAsyncIteration

            for trade in resp.get("trades", []):
                await process_trade(trade, descriptions)

            previous_time = trade["time_init"]
            if total > 100:
                for page in range(0, total, 100):
                    if page in (0, 100):
                        continue
                    resp = await self._state.http.get_trade_history(page, previous_time)
                    resp = resp["response"]
                    for trade in resp.get("trades", []):
                        await process_trade(trade, descriptions)
                    previous_time = trade["time_init"]
                resp = await self._state.http.get_trade_history(page + 100, previous_time)
                resp = resp["response"]
                for trade in resp.get("trades", []):
                    await process_trade(trade, descriptions)
        except StopAsyncIteration:
            return



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_video.proto
# plugin: python-betterproto

from dataclasses import dataclass
from typing import List

import betterproto


@dataclass
class CVideo_ClientGetVideoURL_Request(betterproto.Message):
    video_id: int = betterproto.uint64_field(1)
    client_cellid: int = betterproto.uint32_field(2)


@dataclass
class CVideo_ClientGetVideoURL_Response(betterproto.Message):
    video_id: int = betterproto.uint64_field(1)
    video_url: str = betterproto.string_field(2)


@dataclass
class VideoBookmark(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    playback_position_in_seconds: int = betterproto.uint32_field(2)
    video_track_id: int = betterproto.uint64_field(3)
    audio_track_id: int = betterproto.uint64_field(4)
    timedtext_track_id: int = betterproto.uint64_field(5)
    last_modified: int = betterproto.uint32_field(6)
    hide_from_watch_history: bool = betterproto.bool_field(7)
    hide_from_library: bool = betterproto.bool_field(8)


@dataclass
class CVideo_SetVideoBookmark_Notification(betterproto.Message):
    bookmarks: List["VideoBookmark"] = betterproto.message_field(1)


@dataclass
class CVideo_GetVideoBookmarks_Request(betterproto.Message):
    appids: List[int] = betterproto.uint32_field(1)
    updated_since: int = betterproto.uint32_field(2)


@dataclass
class CVideo_GetVideoBookmarks_Response(betterproto.Message):
    bookmarks: List["VideoBookmark"] = betterproto.message_field(1)


@dataclass
class CVideo_UnlockedH264_Notification(betterproto.Message):
    encryption_key: bytes = betterproto.bytes_field(1)


@dataclass
class CFovasVideo_ClientGetOPFSettings_Request(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    client_cellid: int = betterproto.uint32_field(2)


@dataclass
class CFovasVideo_ClientGetOPFSettings_Response(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    opf_settings: str = betterproto.string_field(2)



# -*- coding: utf-8 -*-

"""
The MIT License (MIT)

Copyright (c) 2015 Rossen Georgiev <rossen@rgp.io>
Copyright (c) 2015-2020 Rapptz
Copyright (c) 2020 James

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import asyncio
import json
import re
from inspect import isawaitable
from operator import attrgetter
from typing import Any, Awaitable, Callable, Iterable, Optional, T, Tuple, Union

import aiohttp

from .enums import EInstanceFlag, EType, ETypeChar, EUniverse

__all__ = (
    "get",
    "find",
    "make_steam64",
    "parse_trade_url_token",
)

PROTOBUF_MASK = 0x80000000
MAX_ASYNCIO_SECONDS = 60 * 60 * 24 * 40
_INVITE_HEX = "0123456789abcdef"
_INVITE_CUSTOM = "bcdfghjkmnpqrtvw"
_INVITE_VALID = f"{_INVITE_HEX}{_INVITE_CUSTOM}"
_INVITE_MAPPING = dict(zip(_INVITE_HEX, _INVITE_CUSTOM))
_INVITE_INVERSE_MAPPING = dict(zip(_INVITE_CUSTOM, _INVITE_HEX))

# from ValvePython/steam

ETypeChars = "".join(type_char.name for type_char in ETypeChar)


def is_proto(emsg: int) -> bool:
    return (int(emsg) & PROTOBUF_MASK) > 0


def set_proto_bit(emsg: int) -> int:
    return int(emsg) | PROTOBUF_MASK


def clear_proto_bit(emsg: int) -> int:
    return int(emsg) & ~PROTOBUF_MASK


def make_steam64(id: Union[int, str] = 0, *args, **kwargs) -> int:
    """Returns a Steam 64-bit ID from various other representations.

    .. code:: python

        make_steam64()  # invalid steam_id
        make_steam64(12345)  # account_id
        make_steam64(12345, is_clan=True)  # makes the account_id into a clan id
        make_steam64('12345')
        make_steam64(id=12345, type='Invalid', universe='Invalid', instance=0)
        make_steam64(103582791429521412)  # steam64
        make_steam64('103582791429521412')
        make_steam64('STEAM_1:0:2')  # steam2
        make_steam64('[g:1:4]')  # steam3
        make_steam64('cv-dgb')  # invite code

    Raises
    ------
    :exc:`TypeError`
        Too many arguments have been given.

    Returns
    -------
    :class:`int`
        The 64-bit Steam ID.
    """

    etype = EType.Invalid
    universe = EUniverse.Invalid
    instance = None
    is_clan = kwargs.pop("is_clan", False)

    if len(args) == 0 and len(kwargs) == 0:
        value = str(id)

        # numeric input
        if value.isdigit():
            value = int(value)

            # 32 bit account id
            if 0 < value < 2 ** 32:
                id = value
                etype = EType.Individual if not is_clan else EType.Clan
                universe = EUniverse.Public
            # 64 bit
            elif value < 2 ** 64:
                id = value & 0xFFFFFFFF
                instance = (value >> 32) & 0xFFFFF
                etype = (value >> 52) & 0xF
                universe = (value >> 56) & 0xFF
            else:
                id = 0

        # textual input e.g. [g:1:4]
        else:
            result = steam2_to_tuple(value) or steam3_to_tuple(value) or invite_code_to_tuple(value)

            if result:
                id, etype, universe, instance = result
            else:
                id = 0

    elif len(args) > 0:
        length = len(args)
        if length == 1:
            etype = args
        elif length == 2:
            etype, universe = args
        elif length == 3:
            etype, universe, instance = args
        else:
            raise TypeError(f"Takes at most 4 arguments ({length} given)")

    if len(kwargs) > 0:
        etype = kwargs.get("type", etype)
        universe = kwargs.get("universe", universe)
        instance = kwargs.get("instance", instance)

    etype = EType.try_value(etype)
    universe = EUniverse.try_value(universe)

    if instance is None:
        instance = 1 if etype in (EType.Individual, EType.GameServer) else 0

    return int(universe) << 56 | int(etype) << 52 | int(instance) << 32 | id


def steam2_to_tuple(value: str) -> Optional[Tuple[int, EType, EUniverse, int]]:
    """
    Parameters
    ----------
    value: :class:`str`
        steam2 e.g. ``STEAM_1:0:1234``.

    Returns
    -------
    Optional[:class:`tuple`]
        e.g. (account_id, type, universe, instance) or ``None``.

    .. note::
        The universe will be always set to ``1``. See :attr:`SteamID.as_steam2`.
    """
    match = re.match(r"STEAM_(?P<universe>\d+)" r":(?P<reminder>[0-1])" r":(?P<id>\d+)", value)

    if not match:
        return None

    steam_32 = (int(match.group("id")) << 1) | int(match.group("reminder"))
    universe = int(match.group("universe"))

    # games before orange box used to incorrectly display universe as 0, we support that
    if universe == 0:
        universe = 1

    return steam_32, EType(1), EUniverse(universe), 1


def steam3_to_tuple(value: str) -> Optional[Tuple[int, EType, EUniverse, int]]:
    """
    Parameters
    ----------
    value: :class:`str`
        steam3 e.g. ``[U:1:1234]``.

    Returns
    -------
    Optional[:class:`tuple`]
        e.g. (account_id, type, universe, instance) or ``None``.
    """
    match = re.match(
        rf"\[(?P<type>[i{ETypeChars}]):"  # type char
        r"(?P<universe>[0-4]):"  # universe
        r"(?P<id>\d{1,10})"  # accountid
        r"(:(?P<instance>\d+))?\]",  # instance
        value,
    )
    if not match:
        return None

    steam_32 = int(match.group("id"))
    universe = EUniverse(int(match.group("universe")))
    typechar = match.group("type").replace("i", "I")
    etype = EType(ETypeChar[typechar])
    instance = match.group("instance")

    if typechar in "gT":
        instance = 0
    elif instance is not None:
        instance = int(instance)
    elif typechar == "L":
        instance = EInstanceFlag.Lobby
    elif typechar == "c":
        instance = EInstanceFlag.Clan
    elif etype in (EType.Individual, EType.GameServer):
        instance = 1
    else:
        instance = 0

    instance = int(instance)

    return steam_32, etype, universe, instance


def invite_code_to_tuple(code: str) -> Optional[Tuple[int, EType, EUniverse, int]]:
    """
    Parameters
    ----------
    code: :class:`str`
        The invite code e.g. ``cv-dgb``

    Returns
    -------
    Optional[:class:`tuple`]
        e.g. (account_id, type, universe, instance) or ``None``.
    """
    match = re.match(
        rf"(https?://s\.team/p/(?P<code1>[\-{_INVITE_VALID}]+))" rf"|(?P<code2>[\-{_INVITE_VALID}]+)", code,
    )
    if not match:
        return None

    code = (match.group("code1") or match.group("code2")).replace("-", "")

    def repl_mapper(x):
        return _INVITE_INVERSE_MAPPING[x.group()]

    steam_32 = int(re.sub(f"[{_INVITE_CUSTOM}]", repl_mapper, code), 16)

    if 0 < steam_32 < 2 ** 32:
        return steam_32, EType(1), EUniverse.Public, 1


async def steam64_from_url(url: str, session: aiohttp.ClientSession = None, timeout: float = 30) -> Optional[int]:
    """Takes a Steam Community url and returns steam64 or None

    .. note::
        Each call makes a http request to steamcommunity.com

    .. note::
        Example URLs
            https://steamcommunity.com/gid/[g:1:4]

            https://steamcommunity.com/gid/103582791429521412

            https://steamcommunity.com/groups/Valve

            https://steamcommunity.com/profiles/[U:1:12]

            https://steamcommunity.com/profiles/76561197960265740

            https://steamcommunity.com/id/johnc

    Parameters
    ----------
    url: :class:`str`
        The Steam community url.
    session: Optional[:class:`aiohttp.ClientSession`]
        The session to make the request with. If
        ``None`` is passed a new one is generated
    timeout: Optional[:class:`float`]
        How long to wait on http request before turning ``None``.

    Returns
    -------
    steam64: Optional[:class:`int`]
        If ``https://steamcommunity.com`` is down or no matching account is found returns ``None``
    """

    search = re.search(
        r"(?P<clean_url>(?:http[s]?://|)(?:www\.|)steamcommunity\.com/"
        r"(?P<type>profiles|id|gid|groups)/(?P<value>.+))",
        str(url).rstrip("/"),
    )

    if search is None:
        return None

    gave_session = bool(session)
    session = session or aiohttp.ClientSession()

    # user profiles
    try:
        if search.group("type") in ("id", "profiles"):
            r = await session.get(search.group("clean_url"), timeout=timeout)
            text = await r.text()
            data_match = re.search("g_rgProfileData\s*=\s*(?P<json>{.*?});\s*", text)

            if data_match:
                data = json.loads(data_match.group("json"))
                return int(data["steamid"])
        # group profiles
        else:
            r = await session.get(search.group("clean_url"), timeout=timeout)
            text = await r.text()
            data_match = re.search(r"OpenGroupChat\(\s*'(?P<steam_id>\d+)'\s*\)", text)

            if data_match:
                return int(data_match.group("steam_id"))
    finally:
        if not gave_session:
            await session.close()


def parse_trade_url_token(url: str) -> Optional[str]:
    """Parses a trade URL for an user's token.

    Parameters
    -----------
    url: :class:`str`
        The URL to search for a token.

    Returns
    -------
    Optional[:class:`str`]
        The found token or ``None`` if the URL doesn't match the regex.
    """
    search = re.search(
        r"(?:http[s]?://|)(?:www.|)steamcommunity.com/tradeoffer/new/\?partner=\d+" r"&token=(?P<token>[\w-]{7,})",
        replace_steam_code(url),
    )
    if search:
        return search.group("token")
    return None


def ainput(prompt: str = "") -> Awaitable[str]:
    loop = asyncio.get_running_loop()
    return loop.run_in_executor(None, input, prompt)


# everything below here is directly from discord.py's utils
# https://github.com/rapptz/discord.py/blob/master/discord/utils.py


def find(predicate: Callable[..., bool], iterable: Iterable) -> Optional[Any]:
    """A helper to return the first element found in the sequence.

    Parameters
    -----------
    predicate: Callable[..., bool]
        A function that returns a boolean.
    iterable: Iterable
        The iterable to search through.

    Returns
    -------
    Optional[Any]
        The first element from the ``iterable``
        for which the ``predicate`` returns ``True``
        or ``None`` if no matching element was found.
    """

    for element in iterable:
        if predicate(element):
            return element
    return None


def get(iterable: Iterable[T], **attrs) -> Optional[T]:
    r"""A helper that returns the first element in the iterable that meets
    all the traits passed in ``attrs``. This is an alternative for
    :func:`utils.find`.

    Parameters
    -----------
    iterable: Iterable[T]
        An iterable to search through.
    \*\*attrs
        Keyword arguments that denote attributes to match.

    Returns
    -------
    Optional[T]
        The first element from the ``iterable``
        which matches all the traits passed in ``attrs``
        or ``None`` if no matching element was found.
    """

    # global -> local
    _all = all
    attrget = attrgetter

    # Special case the single element call
    if len(attrs) == 1:
        k, v = attrs.popitem()
        pred = attrget(k.replace("__", "."))
        for elem in iterable:
            if pred(elem) == v:
                return elem
        return None

    converted = [(attrget(attr.replace("__", ".")), value) for attr, value in attrs.items()]

    for elem in iterable:
        if _all(pred(elem) == value for pred, value in converted):
            return elem
    return None


async def maybe_coroutine(func: Callable[..., Union[Any, Awaitable]], *args, **kwargs) -> Any:
    value = func(*args, **kwargs)
    if isawaitable(value):
        return await value
    return value


def replace_steam_code(s: str) -> str:
    steam_code = [  # TODO try and find a list for these?
        ("&quot;", '"'),
        ("&gt;", ">"),
        ("&lt;", "<"),
        ("&amp;", "&"),
    ]
    for code in steam_code:
        s = s.replace(*code)
    return s



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_webui_friends.proto
# plugin: python-betterproto

from dataclasses import dataclass
from typing import TYPE_CHECKING, List

import betterproto

if TYPE_CHECKING:
    from .steammessages_base import CCDDBAppDetailCommon, CClanMatchEventByRange, CMsgIPAddress
    from .steammessages_clientserver_friends import CMsgClientFriendsList


@dataclass
class CCommunity_GetApps_Request(betterproto.Message):
    appids: List[int] = betterproto.int32_field(1)
    language: int = betterproto.uint32_field(2)


@dataclass
class CCommunity_GetApps_Response(betterproto.Message):
    apps: List["CCDDBAppDetailCommon"] = betterproto.message_field(1)


@dataclass
class CCommunity_GetAppRichPresenceLocalization_Request(betterproto.Message):
    appid: int = betterproto.int32_field(1)
    language: str = betterproto.string_field(2)


@dataclass
class CCommunity_GetAppRichPresenceLocalization_Response(betterproto.Message):
    appid: int = betterproto.int32_field(1)
    token_lists: List["CCommunity_GetAppRichPresenceLocalization_Response_TokenList"] = betterproto.message_field(2)


@dataclass
class CCommunity_GetAppRichPresenceLocalization_Response_Token(betterproto.Message):
    name: str = betterproto.string_field(1)
    value: str = betterproto.string_field(2)


@dataclass
class CCommunity_GetAppRichPresenceLocalization_Response_TokenList(betterproto.Message):
    language: str = betterproto.string_field(1)
    tokens: List["CCommunity_GetAppRichPresenceLocalization_Response_Token"] = betterproto.message_field(2)


@dataclass
class CCommunity_Comment(betterproto.Message):
    gidcomment: float = betterproto.fixed64_field(1)
    steamid: float = betterproto.fixed64_field(2)
    timestamp: int = betterproto.uint32_field(3)
    text: str = betterproto.string_field(4)
    upvotes: int = betterproto.int32_field(5)
    hidden: bool = betterproto.bool_field(6)
    hidden_by_user: bool = betterproto.bool_field(7)
    deleted: bool = betterproto.bool_field(8)
    ipaddress: "CMsgIPAddress" = betterproto.message_field(9)
    total_hidden: int = betterproto.int32_field(10)
    upvoted_by_user: bool = betterproto.bool_field(11)


@dataclass
class CCommunity_GetCommentThread_Response(betterproto.Message):
    comments: List["CCommunity_Comment"] = betterproto.message_field(1)
    deleted_comments: List["CCommunity_Comment"] = betterproto.message_field(2)
    steamid: float = betterproto.fixed64_field(3)
    commentthreadid: float = betterproto.fixed64_field(4)
    start: int = betterproto.int32_field(5)
    count: int = betterproto.int32_field(6)
    total_count: int = betterproto.int32_field(7)
    upvotes: int = betterproto.int32_field(8)
    upvoters: List[int] = betterproto.uint32_field(9)
    user_subscribed: bool = betterproto.bool_field(10)
    user_upvoted: bool = betterproto.bool_field(11)
    answer_commentid: float = betterproto.fixed64_field(12)
    answer_actor: int = betterproto.uint32_field(13)
    answer_actor_rank: int = betterproto.int32_field(14)
    can_post: bool = betterproto.bool_field(15)


@dataclass
class CCommunity_PostCommentToThread_Response(betterproto.Message):
    gidcomment: float = betterproto.fixed64_field(1)
    commentthreadid: float = betterproto.fixed64_field(2)
    count: int = betterproto.int32_field(3)
    upvotes: int = betterproto.int32_field(4)


@dataclass
class CCommunity_DeleteCommentFromThread_Response(betterproto.Message):
    pass


@dataclass
class CCommunity_RateCommentThread_Response(betterproto.Message):
    gidcomment: int = betterproto.uint64_field(1)
    commentthreadid: int = betterproto.uint64_field(2)
    count: int = betterproto.uint32_field(3)
    upvotes: int = betterproto.uint32_field(4)
    has_upvoted: bool = betterproto.bool_field(5)


@dataclass
class CCommunity_GetCommentThreadRatings_Response(betterproto.Message):
    commentthreadid: int = betterproto.uint64_field(1)
    gidcomment: int = betterproto.uint64_field(2)
    upvotes: int = betterproto.uint32_field(3)
    has_upvoted: bool = betterproto.bool_field(4)
    upvoter_accountids: List[int] = betterproto.uint32_field(5)


@dataclass
class CCommunity_RateClanAnnouncement_Response(betterproto.Message):
    pass


@dataclass
class CCommunity_GetClanAnnouncementVoteForUser_Response(betterproto.Message):
    voted_up: bool = betterproto.bool_field(1)
    voted_down: bool = betterproto.bool_field(2)


@dataclass
class CAppPriority(betterproto.Message):
    priority: int = betterproto.uint32_field(1)
    appid: List[int] = betterproto.uint32_field(2)


@dataclass
class CCommunity_GetUserPartnerEventNews_Response(betterproto.Message):
    results: List["CClanMatchEventByRange"] = betterproto.message_field(1)


@dataclass
class CCommunity_PartnerEventResult(betterproto.Message):
    clanid: int = betterproto.uint32_field(1)
    event_gid: float = betterproto.fixed64_field(2)
    announcement_gid: float = betterproto.fixed64_field(3)
    appid: int = betterproto.uint32_field(4)
    possible_takeover: bool = betterproto.bool_field(5)
    rtime32_last_modified: int = betterproto.uint32_field(6)
    user_app_priority: int = betterproto.int32_field(7)


@dataclass
class CCommunity_GetBestEventsForUser_Response(betterproto.Message):
    results: List["CCommunity_PartnerEventResult"] = betterproto.message_field(1)


@dataclass
class CCommunity_ClearUserPartnerEventsAppPriorities_Response(betterproto.Message):
    pass


@dataclass
class CCommunity_PartnerEventsAppPriority(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    user_app_priority: int = betterproto.int32_field(2)


@dataclass
class CCommunity_GetUserPartnerEventsAppPriorities_Response(betterproto.Message):
    priorities: List["CCommunity_PartnerEventsAppPriority"] = betterproto.message_field(1)


@dataclass
class CCommunity_ClearSinglePartnerEventsAppPriority_Response(betterproto.Message):
    pass


@dataclass
class CCommunity_PartnerEventsShowMoreForApp_Response(betterproto.Message):
    pass


@dataclass
class CCommunity_PartnerEventsShowLessForApp_Response(betterproto.Message):
    pass


@dataclass
class CCommunity_MarkPartnerEventsForUser_Request_PartnerEventMarking(betterproto.Message):
    clanid: int = betterproto.uint32_field(1)
    event_gid: float = betterproto.fixed64_field(2)
    display_location: int = betterproto.int32_field(3)
    mark_shown: bool = betterproto.bool_field(4)
    mark_read: bool = betterproto.bool_field(5)


@dataclass
class CCommunity_MarkPartnerEventsForUser_Response(betterproto.Message):
    pass


@dataclass
class CCommunity_GetUserPartnerEventViewStatus_Response(betterproto.Message):
    events: List["CCommunity_GetUserPartnerEventViewStatus_Response_PartnerEvent"] = betterproto.message_field(1)


@dataclass
class CCommunity_GetUserPartnerEventViewStatus_Response_PartnerEvent(betterproto.Message):
    event_gid: float = betterproto.fixed64_field(1)
    last_shown_time: int = betterproto.uint32_field(2)
    last_read_time: int = betterproto.uint32_field(3)
    clan_account_id: int = betterproto.uint32_field(4)


@dataclass
class ProfileItem(betterproto.Message):
    communityitemid: int = betterproto.uint64_field(1)
    image_small: str = betterproto.string_field(2)
    image_large: str = betterproto.string_field(3)
    name: str = betterproto.string_field(4)
    item_title: str = betterproto.string_field(5)
    item_description: str = betterproto.string_field(6)
    appid: int = betterproto.uint32_field(7)
    item_type: int = betterproto.uint32_field(8)
    item_class: int = betterproto.uint32_field(9)
    movie_webm: str = betterproto.string_field(10)
    movie_mp4: str = betterproto.string_field(11)


@dataclass
class CPlayer_GetProfileBackground_Response(betterproto.Message):
    profile_background: "ProfileItem" = betterproto.message_field(1)


@dataclass
class CPlayer_SetProfileBackground_Response(betterproto.Message):
    pass


@dataclass
class CPlayer_GetMiniProfileBackground_Response(betterproto.Message):
    profile_background: "ProfileItem" = betterproto.message_field(1)


@dataclass
class CPlayer_SetMiniProfileBackground_Response(betterproto.Message):
    pass


@dataclass
class CPlayer_GetAvatarFrame_Response(betterproto.Message):
    avatar_frame: "ProfileItem" = betterproto.message_field(1)


@dataclass
class CPlayer_SetAvatarFrame_Response(betterproto.Message):
    pass


@dataclass
class CPlayer_GetAnimatedAvatar_Response(betterproto.Message):
    avatar: "ProfileItem" = betterproto.message_field(1)


@dataclass
class CPlayer_SetAnimatedAvatar_Response(betterproto.Message):
    pass


@dataclass
class CPlayer_GetProfileItemsOwned_Response(betterproto.Message):
    profile_backgrounds: List["ProfileItem"] = betterproto.message_field(1)
    mini_profile_backgrounds: List["ProfileItem"] = betterproto.message_field(2)
    avatar_frames: List["ProfileItem"] = betterproto.message_field(3)
    animated_avatars: List["ProfileItem"] = betterproto.message_field(4)


@dataclass
class CPlayer_GetProfileItemsEquipped_Response(betterproto.Message):
    profile_background: "ProfileItem" = betterproto.message_field(1)
    mini_profile_background: "ProfileItem" = betterproto.message_field(2)
    avatar_frame: "ProfileItem" = betterproto.message_field(3)
    animated_avatar: "ProfileItem" = betterproto.message_field(4)


@dataclass
class CWebRTCClient_InitiateWebRTCConnection_Request(betterproto.Message):
    sdp: str = betterproto.string_field(1)


@dataclass
class CWebRTCClient_InitiateWebRTCConnection_Response(betterproto.Message):
    remote_description: str = betterproto.string_field(1)


@dataclass
class CWebRTC_WebRTCSessionConnected_Notification(betterproto.Message):
    ssrc: int = betterproto.uint32_field(1)
    client_ip: int = betterproto.uint32_field(2)
    client_port: int = betterproto.uint32_field(3)
    server_ip: int = betterproto.uint32_field(4)
    server_port: int = betterproto.uint32_field(5)


@dataclass
class CWebRTC_WebRTCUpdateRemoteDescription_Notification(betterproto.Message):
    remote_description: str = betterproto.string_field(1)
    remote_description_version: int = betterproto.uint64_field(2)
    ssrcs_to_accountids: List[
        "CWebRTC_WebRTCUpdateRemoteDescription_Notification_CSSRCToAccountIDMapping"
    ] = betterproto.message_field(3)


@dataclass
class CWebRTC_WebRTCUpdateRemoteDescription_Notification_CSSRCToAccountIDMapping(betterproto.Message):
    ssrc: int = betterproto.uint32_field(1)
    accountid: int = betterproto.uint32_field(2)


@dataclass
class CWebRTCClient_AcknowledgeUpdatedRemoteDescription_Request(betterproto.Message):
    ip_webrtc_server: int = betterproto.uint32_field(1)
    port_webrtc_server: int = betterproto.uint32_field(2)
    ip_webrtc_session_client: int = betterproto.uint32_field(3)
    port_webrtc_session_client: int = betterproto.uint32_field(4)
    remote_description_version: int = betterproto.uint64_field(5)


@dataclass
class CWebRTCClient_AcknowledgeUpdatedRemoteDescription_Response(betterproto.Message):
    pass


@dataclass
class CVoiceChat_RequestOneOnOneChat_Request(betterproto.Message):
    steamid_partner: float = betterproto.fixed64_field(1)


@dataclass
class CVoiceChat_RequestOneOnOneChat_Response(betterproto.Message):
    voice_chatid: float = betterproto.fixed64_field(1)


@dataclass
class CVoiceChat_OneOnOneChatRequested_Notification(betterproto.Message):
    voice_chatid: float = betterproto.fixed64_field(1)
    steamid_partner: float = betterproto.fixed64_field(2)


@dataclass
class CVoiceChat_AnswerOneOnOneChat_Request(betterproto.Message):
    voice_chatid: float = betterproto.fixed64_field(1)
    steamid_partner: float = betterproto.fixed64_field(2)
    accepted_request: bool = betterproto.bool_field(3)


@dataclass
class CVoiceChat_AnswerOneOnOneChat_Response(betterproto.Message):
    pass


@dataclass
class CVoiceChat_OneOnOneChatRequestResponse_Notification(betterproto.Message):
    voicechat_id: float = betterproto.fixed64_field(1)
    steamid_partner: float = betterproto.fixed64_field(2)
    accepted_request: bool = betterproto.bool_field(3)


@dataclass
class CVoiceChat_EndOneOnOneChat_Request(betterproto.Message):
    steamid_partner: float = betterproto.fixed64_field(1)


@dataclass
class CVoiceChat_EndOneOnOneChat_Response(betterproto.Message):
    pass


@dataclass
class CVoiceChat_LeaveOneOnOneChat_Request(betterproto.Message):
    steamid_partner: float = betterproto.fixed64_field(1)
    voice_chatid: float = betterproto.fixed64_field(2)


@dataclass
class CVoiceChat_LeaveOneOnOneChat_Response(betterproto.Message):
    pass


@dataclass
class CVoiceChat_UserJoinedVoiceChat_Notification(betterproto.Message):
    voice_chatid: float = betterproto.fixed64_field(1)
    user_steamid: float = betterproto.fixed64_field(2)
    chatid: int = betterproto.uint64_field(3)
    one_on_one_steamid_lower: float = betterproto.fixed64_field(4)
    one_on_one_steamid_higher: float = betterproto.fixed64_field(5)
    chat_group_id: int = betterproto.uint64_field(6)
    user_sessionid: int = betterproto.uint32_field(7)


@dataclass
class CVoiceChat_UserVoiceStatus_Notification(betterproto.Message):
    voice_chatid: float = betterproto.fixed64_field(1)
    user_steamid: float = betterproto.fixed64_field(2)
    user_muted_mic_locally: bool = betterproto.bool_field(3)
    user_muted_output_locally: bool = betterproto.bool_field(4)
    user_has_no_mic_for_session: bool = betterproto.bool_field(5)
    user_webaudio_sample_rate: int = betterproto.int32_field(6)


@dataclass
class CVoiceChat_AllMembersStatus_Notification(betterproto.Message):
    voice_chatid: float = betterproto.fixed64_field(1)
    users: List["CVoiceChat_UserVoiceStatus_Notification"] = betterproto.message_field(2)


@dataclass
class CVoiceChat_UpdateVoiceChatWebRTCData_Request(betterproto.Message):
    voice_chatid: float = betterproto.fixed64_field(1)
    ip_webrtc_server: int = betterproto.uint32_field(2)
    port_webrtc_server: int = betterproto.uint32_field(3)
    ip_webrtc_client: int = betterproto.uint32_field(4)
    port_webrtc_client: int = betterproto.uint32_field(5)
    ssrc_my_sending_stream: int = betterproto.uint32_field(6)
    user_agent: str = betterproto.string_field(7)
    has_audio_worklets_support: bool = betterproto.bool_field(8)


@dataclass
class CVoiceChat_UpdateVoiceChatWebRTCData_Response(betterproto.Message):
    send_client_voice_logs: bool = betterproto.bool_field(1)


@dataclass
class CVoiceChat_UploadClientVoiceChatLogs_Request(betterproto.Message):
    voice_chatid: float = betterproto.fixed64_field(1)
    client_voice_logs_new_lines: str = betterproto.string_field(2)


@dataclass
class CVoiceChat_UploadClientVoiceChatLogs_Response(betterproto.Message):
    pass


@dataclass
class CVoiceChat_LeaveVoiceChat_Response(betterproto.Message):
    pass


@dataclass
class CVoiceChat_UserLeftVoiceChat_Notification(betterproto.Message):
    voice_chatid: float = betterproto.fixed64_field(1)
    user_steamid: float = betterproto.fixed64_field(2)
    chatid: int = betterproto.uint64_field(3)
    one_on_one_steamid_lower: float = betterproto.fixed64_field(4)
    one_on_one_steamid_higher: float = betterproto.fixed64_field(5)
    chat_group_id: int = betterproto.uint64_field(6)
    user_sessionid: int = betterproto.uint32_field(7)


@dataclass
class CVoiceChat_VoiceChatEnded_Notification(betterproto.Message):
    voice_chatid: float = betterproto.fixed64_field(1)
    one_on_one_steamid_lower: float = betterproto.fixed64_field(2)
    one_on_one_steamid_higher: float = betterproto.fixed64_field(3)
    chatid: int = betterproto.uint64_field(4)
    chat_group_id: int = betterproto.uint64_field(5)


@dataclass
class CSteamTV_CreateBroadcastChannel_Response(betterproto.Message):
    broadcast_channel_id: float = betterproto.fixed64_field(1)


@dataclass
class CSteamTV_GetBroadcastChannelID_Response(betterproto.Message):
    broadcast_channel_id: float = betterproto.fixed64_field(1)
    unique_name: str = betterproto.string_field(2)
    steamid: float = betterproto.fixed64_field(3)


@dataclass
class CSteamTV_SetBroadcastChannelProfile_Response(betterproto.Message):
    pass


@dataclass
class CSteamTV_GetBroadcastChannelProfile_Response(betterproto.Message):
    unique_name: str = betterproto.string_field(1)
    owner_steamid: float = betterproto.fixed64_field(2)
    name: str = betterproto.string_field(3)
    language: str = betterproto.string_field(4)
    headline: str = betterproto.string_field(5)
    summary: str = betterproto.string_field(6)
    schedule: str = betterproto.string_field(7)
    rules: str = betterproto.string_field(8)
    panels: str = betterproto.string_field(9)
    is_partnered: bool = betterproto.bool_field(10)


@dataclass
class CSteamTV_SetBroadcastChannelImage_Response(betterproto.Message):
    replace_image_hash: str = betterproto.string_field(1)


@dataclass
class CSteamTV_GetBroadcastChannelImages_Response(betterproto.Message):
    images: List["CSteamTV_GetBroadcastChannelImages_Response_Images"] = betterproto.message_field(1)


@dataclass
class CSteamTV_GetBroadcastChannelImages_Response_Images(betterproto.Message):
    image_type: int = betterproto.int32_field(1)
    image_path: str = betterproto.string_field(2)
    image_index: int = betterproto.uint32_field(3)


@dataclass
class CSteamTV_GetBroadcastChannelLinks_Response(betterproto.Message):
    links: List["CSteamTV_GetBroadcastChannelLinks_Response_Links"] = betterproto.message_field(1)


@dataclass
class CSteamTV_GetBroadcastChannelLinks_Response_Links(betterproto.Message):
    link_index: int = betterproto.uint32_field(1)
    url: str = betterproto.string_field(2)
    link_description: str = betterproto.string_field(3)
    left: int = betterproto.uint32_field(4)
    top: int = betterproto.uint32_field(5)
    width: int = betterproto.uint32_field(6)
    height: int = betterproto.uint32_field(7)


@dataclass
class CSteamTV_SetBroadcastChannelLinkRegions_Request_Links(betterproto.Message):
    link_index: int = betterproto.uint32_field(1)
    url: str = betterproto.string_field(2)
    link_description: str = betterproto.string_field(3)
    left: int = betterproto.uint32_field(4)
    top: int = betterproto.uint32_field(5)
    width: int = betterproto.uint32_field(6)
    height: int = betterproto.uint32_field(7)


@dataclass
class CSteamTV_SetBroadcastChannelLinkRegions_Response(betterproto.Message):
    pass


@dataclass
class CSteamTV_GetBroadcastChannelStatus_Response(betterproto.Message):
    is_live: bool = betterproto.bool_field(1)
    is_disabled: bool = betterproto.bool_field(2)
    appid: int = betterproto.uint32_field(3)
    viewers: int = betterproto.uint64_field(4)
    views: int = betterproto.uint64_field(5)
    broadcaster_steamid: float = betterproto.fixed64_field(6)
    thumbnail_url: str = betterproto.string_field(7)
    followers: int = betterproto.uint64_field(8)
    subscribers: int = betterproto.uint64_field(9)
    unique_name: str = betterproto.string_field(10)
    broadcast_session_id: int = betterproto.uint64_field(11)


@dataclass
class GetBroadcastChannelEntry(betterproto.Message):
    broadcast_channel_id: float = betterproto.fixed64_field(1)
    unique_name: str = betterproto.string_field(2)
    name: str = betterproto.string_field(3)
    appid: int = betterproto.uint32_field(4)
    viewers: int = betterproto.uint64_field(5)
    views: int = betterproto.uint64_field(6)
    thumbnail_url: str = betterproto.string_field(7)
    followers: int = betterproto.uint64_field(8)
    headline: str = betterproto.string_field(9)
    avatar_url: str = betterproto.string_field(10)
    broadcaster_steamid: float = betterproto.fixed64_field(11)
    subscribers: int = betterproto.uint64_field(12)
    background_url: str = betterproto.string_field(13)
    is_featured: bool = betterproto.bool_field(14)
    is_disabled: bool = betterproto.bool_field(15)
    is_live: bool = betterproto.bool_field(16)
    language: str = betterproto.string_field(17)
    reports: int = betterproto.uint32_field(18)
    is_partnered: bool = betterproto.bool_field(19)


@dataclass
class CSteamTV_GetFollowedChannels_Response(betterproto.Message):
    results: List["GetBroadcastChannelEntry"] = betterproto.message_field(1)


@dataclass
class CSteamTV_GetSubscribedChannels_Response(betterproto.Message):
    results: List["GetBroadcastChannelEntry"] = betterproto.message_field(1)


@dataclass
class CSteamTV_FollowBroadcastChannel_Response(betterproto.Message):
    is_followed: bool = betterproto.bool_field(1)


@dataclass
class CSteamTV_SubscribeBroadcastChannel_Response(betterproto.Message):
    is_subscribed: bool = betterproto.bool_field(1)


@dataclass
class CSteamTV_ReportBroadcastChannel_Response(betterproto.Message):
    pass


@dataclass
class CSteamTV_GetBroadcastChannelInteraction_Response(betterproto.Message):
    is_followed: bool = betterproto.bool_field(1)
    is_subscribed: bool = betterproto.bool_field(2)


@dataclass
class CSteamTV_Game(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    name: str = betterproto.string_field(2)
    image: str = betterproto.string_field(3)
    viewers: int = betterproto.uint64_field(4)
    channels: List["GetBroadcastChannelEntry"] = betterproto.message_field(5)
    release_date: str = betterproto.string_field(6)
    developer: str = betterproto.string_field(7)
    publisher: str = betterproto.string_field(8)


@dataclass
class CSteamTV_GetGames_Response(betterproto.Message):
    results: List["CSteamTV_Game"] = betterproto.message_field(1)


@dataclass
class CSteamTV_GetChannels_Response(betterproto.Message):
    results: List["GetBroadcastChannelEntry"] = betterproto.message_field(1)


@dataclass
class CSteamTV_GetBroadcastChannelBroadcasters_Response(betterproto.Message):
    broadcasters: List["CSteamTV_GetBroadcastChannelBroadcasters_Response_Broadcaster"] = betterproto.message_field(1)


@dataclass
class CSteamTV_GetBroadcastChannelBroadcasters_Response_Broadcaster(betterproto.Message):
    steamid: float = betterproto.fixed64_field(1)
    name: str = betterproto.string_field(2)
    rtmp_token: str = betterproto.string_field(3)


@dataclass
class CSteamTV_ChatBan(betterproto.Message):
    issuer_steamid: float = betterproto.fixed64_field(1)
    chatter_steamid: float = betterproto.fixed64_field(2)
    time_expires: str = betterproto.string_field(3)
    permanent: bool = betterproto.bool_field(4)
    name: str = betterproto.string_field(5)


@dataclass
class CSteamTV_AddChatBan_Request(betterproto.Message):
    broadcast_channel_id: float = betterproto.fixed64_field(1)
    chatter_steamid: float = betterproto.fixed64_field(2)
    duration: int = betterproto.uint32_field(3)
    permanent: bool = betterproto.bool_field(4)
    undo: bool = betterproto.bool_field(5)


@dataclass
class CSteamTV_AddChatBan_Response(betterproto.Message):
    pass


@dataclass
class CSteamTV_GetChatBans_Response(betterproto.Message):
    results: List["CSteamTV_ChatBan"] = betterproto.message_field(1)


@dataclass
class CSteamTV_AddChatModerator_Request(betterproto.Message):
    broadcast_channel_id: float = betterproto.fixed64_field(1)
    moderator_steamid: float = betterproto.fixed64_field(2)
    undo: bool = betterproto.bool_field(3)


@dataclass
class CSteamTV_AddChatModerator_Response(betterproto.Message):
    pass


@dataclass
class CSteamTV_GetChatModerators_Request(betterproto.Message):
    broadcast_channel_id: float = betterproto.fixed64_field(1)


@dataclass
class CSteamTV_ChatModerator(betterproto.Message):
    steamid: float = betterproto.fixed64_field(1)
    name: str = betterproto.string_field(2)


@dataclass
class CSteamTV_GetChatModerators_Response(betterproto.Message):
    results: List["CSteamTV_ChatModerator"] = betterproto.message_field(1)


@dataclass
class CSteamTV_AddWordBan_Response(betterproto.Message):
    pass


@dataclass
class CSteamTV_GetWordBans_Response(betterproto.Message):
    results: List[str] = betterproto.string_field(1)


@dataclass
class CSteamTV_JoinChat_Request(betterproto.Message):
    broadcast_channel_id: float = betterproto.fixed64_field(1)


@dataclass
class CSteamTV_JoinChat_Response(betterproto.Message):
    chat_id: float = betterproto.fixed64_field(1)
    view_url_template: str = betterproto.string_field(2)
    flair_group_ids: List[int] = betterproto.uint64_field(3)


@dataclass
class CSteamTV_Search_Response(betterproto.Message):
    results: List["GetBroadcastChannelEntry"] = betterproto.message_field(1)


@dataclass
class CSteamTV_GetSteamTVUserSettings_Response(betterproto.Message):
    stream_live_email: bool = betterproto.bool_field(1)
    stream_live_notification: bool = betterproto.bool_field(2)


@dataclass
class CSteamTV_SetSteamTVUserSettings_Response(betterproto.Message):
    pass


@dataclass
class CSteamTV_GetMyBroadcastChannels_Response(betterproto.Message):
    results: List["GetBroadcastChannelEntry"] = betterproto.message_field(1)


@dataclass
class CSteamTV_HomePageTemplate_Takeover(betterproto.Message):
    broadcasts: List["GetBroadcastChannelEntry"] = betterproto.message_field(1)


@dataclass
class CSteamTV_HomePageTemplate_SingleGame(betterproto.Message):
    broadcasts: List["GetBroadcastChannelEntry"] = betterproto.message_field(1)
    appid: int = betterproto.uint32_field(2)
    title: str = betterproto.string_field(3)


@dataclass
class GameListEntry(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    game_name: str = betterproto.string_field(2)
    broadcast: "GetBroadcastChannelEntry" = betterproto.message_field(3)


@dataclass
class CSteamTV_HomePageTemplate_GameList(betterproto.Message):
    entries: List["GameListEntry"] = betterproto.message_field(1)
    title: str = betterproto.string_field(2)


@dataclass
class CSteamTV_HomePageTemplate_QuickExplore(betterproto.Message):
    broadcasts: List["GetBroadcastChannelEntry"] = betterproto.message_field(1)
    title: str = betterproto.string_field(2)


@dataclass
class CSteamTV_HomePageTemplate_ConveyorBelt(betterproto.Message):
    broadcasts: List["GetBroadcastChannelEntry"] = betterproto.message_field(1)
    title: str = betterproto.string_field(2)


@dataclass
class CSteamTV_HomePageTemplate_WatchParty(betterproto.Message):
    broadcast: "GetBroadcastChannelEntry" = betterproto.message_field(1)
    title: str = betterproto.string_field(2)
    chat_group_id: int = betterproto.uint64_field(3)


@dataclass
class CSteamTV_HomePageTemplate_Developer(betterproto.Message):
    broadcast: "GetBroadcastChannelEntry" = betterproto.message_field(1)
    title: str = betterproto.string_field(2)


@dataclass
class CSteamTV_HomePageTemplate_Event(betterproto.Message):
    title: str = betterproto.string_field(1)


@dataclass
class CSteamTV_HomePageContentRow(betterproto.Message):
    template_type: int = betterproto.int32_field(1)
    takeover: "CSteamTV_HomePageTemplate_Takeover" = betterproto.message_field(2)
    single_game: "CSteamTV_HomePageTemplate_SingleGame" = betterproto.message_field(3)
    game_list: "CSteamTV_HomePageTemplate_GameList" = betterproto.message_field(4)
    quick_explore: "CSteamTV_HomePageTemplate_QuickExplore" = betterproto.message_field(5)
    conveyor_belt: "CSteamTV_HomePageTemplate_ConveyorBelt" = betterproto.message_field(6)
    watch_party: "CSteamTV_HomePageTemplate_WatchParty" = betterproto.message_field(7)
    developer: "CSteamTV_HomePageTemplate_Developer" = betterproto.message_field(8)
    event: "CSteamTV_HomePageTemplate_Event" = betterproto.message_field(9)


@dataclass
class CSteamTV_GetHomePageContents_Response(betterproto.Message):
    rows: List["CSteamTV_HomePageContentRow"] = betterproto.message_field(1)


@dataclass
class CSteamTV_BroadcastClipInfo(betterproto.Message):
    broadcast_clip_id: int = betterproto.uint64_field(1)
    channel_id: int = betterproto.uint64_field(2)
    app_id: int = betterproto.uint32_field(3)
    broadcaster_steamid: float = betterproto.fixed64_field(4)
    creator_steamid: float = betterproto.fixed64_field(5)
    video_description: str = betterproto.string_field(6)
    live_time: int = betterproto.uint32_field(7)
    length_ms: int = betterproto.uint32_field(8)
    thumbnail_path: str = betterproto.string_field(9)


@dataclass
class CSteamTV_GetBroadcastChannelClips_Response(betterproto.Message):
    clips: List["CSteamTV_BroadcastClipInfo"] = betterproto.message_field(1)
    thumbnail_host: str = betterproto.string_field(2)


@dataclass
class CFriendsListCategory(betterproto.Message):
    groupid: int = betterproto.uint32_field(1)
    name: str = betterproto.string_field(2)
    accountid_members: List[int] = betterproto.uint32_field(3)


@dataclass
class CFriendsList_GetCategories_Request(betterproto.Message):
    pass


@dataclass
class CFriendsList_GetCategories_Response(betterproto.Message):
    categories: List["CFriendsListCategory"] = betterproto.message_field(1)


@dataclass
class CFriendsListFavoriteEntry(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)
    clanid: int = betterproto.uint32_field(2)
    chat_group_id: int = betterproto.uint64_field(3)


@dataclass
class CFriendsList_GetFavorites_Request(betterproto.Message):
    pass


@dataclass
class CFriendsList_GetFavorites_Response(betterproto.Message):
    favorites: List["CFriendsListFavoriteEntry"] = betterproto.message_field(1)


@dataclass
class CFriendsList_SetFavorites_Request(betterproto.Message):
    favorites: List["CFriendsListFavoriteEntry"] = betterproto.message_field(1)


@dataclass
class CFriendsList_SetFavorites_Response(betterproto.Message):
    pass


@dataclass
class CFriendsList_FavoritesChanged_Notification(betterproto.Message):
    favorites: List["CFriendsListFavoriteEntry"] = betterproto.message_field(1)


@dataclass
class CFriendsList_GetFriendsList_Request(betterproto.Message):
    pass


@dataclass
class CFriendsList_GetFriendsList_Response(betterproto.Message):
    friends_list: "CMsgClientFriendsList" = betterproto.message_field(1)


@dataclass
class CClan_RespondToClanInvite_Request(betterproto.Message):
    steamid: float = betterproto.fixed64_field(1)
    accept: bool = betterproto.bool_field(2)


@dataclass
class CClan_RespondToClanInvite_Response(betterproto.Message):
    pass


@dataclass
class CProductImpressionsFromClient_Notification(betterproto.Message):
    impressions: List["CProductImpressionsFromClient_Notification_Impression"] = betterproto.message_field(1)


@dataclass
class CProductImpressionsFromClient_Notification_Impression(betterproto.Message):
    type: int = betterproto.int32_field(1)
    appid: int = betterproto.uint32_field(2)
    num_impressions: int = betterproto.uint32_field(3)


@dataclass
class NotImplemented(betterproto.Message):
    pass



# -*- coding: utf-8 -*-

import re

from setuptools import setup

with open("requirements.txt") as f:
    requirements = f.read().splitlines()
    f.close()

with open("steam/__init__.py") as f:
    search = re.search(r'^__version__\s*=\s*"([^"]*)"', f.read(), re.MULTILINE)

if search is None:
    raise RuntimeError("Version is not set")

version = search.group(1)

if version.endswith(("a", "b")) or "rc" in version:
    # try to find out the commit hash if checked out from git, and append
    # it to __version__ (since we use this value from setup.py, it gets
    # automatically propagated to an installed copy as well)
    try:
        import subprocess

        out = subprocess.getoutput("git rev-list --count HEAD")
        if out:
            version = f"{version}{out.strip()}"
        out = subprocess.getoutput("git rev-parse --short HEAD")
        if out:
            version = f"{version}+g{out.strip()}"
    except Exception:
        pass

with open("README.md") as f:
    readme = f.read()

extras_require = {"docs": ["sphinx==3.0.1", "sphinxcontrib_trio==1.1.1", "sphinxcontrib-websupport",]}

setup(
    name="steamio",
    author="Gobot1234",
    url="https://github.com/Gobot1234/steam.py",
    project_urls={
        "Documentation": "https://steampy.readthedocs.io/en/latest",
        "Issue tracker": "https://github.com/Gobot1234/steam.py/issues",
    },
    version=version,
    packages=["steam", "steam.protobufs", "steam.ext.commands",],
    license="MIT",
    description="A Python wrapper for the Steam API",
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    install_requires=requirements,
    extras_require=extras_require,
    python_requires=">=3.7",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
